#' Low flow to RI based on Pearson type III statistics
#' @param q_cfs flow in cubic feet per second
#' @importFrom smwrBase plpearsonIII
calc_RI_low <- function(q_cfs){

  # calculate exceedance probability for flow
  lp3_stat <- smwrBase::plpearsonIII(
    q = q_cfs,
    mean =  logqmean_low,
    sd =  logqsd_low,
    skew = logqskew_low)

  1/lp3_stat # calc return interval
}


#' Convert peak q to surv
#'
#' @param q_cfs peak flow in cubic feet per second
#' @param era which era to run
#'
#' @noRd

calc_e2f_mult <- function(q_cfs, era = c("hist", "curr", "mid", "late")){

  flow_adjustment <- switch( # climate-based flow modifier
    era,
    hist = 1, # if run_era == "hist", return 1
    curr = 1,
    mid = peakq_change_mid,
    late = peakq_change_late)

  # reciprocal of adjustment factor used in deterministic code
  # used to "undo" deterministic model climate adjustment
  det_recipr <- calc_e2f_surv(peak_q_median)/
    calc_e2f_surv(peak_q_median*flow_adjustment)

  # change in survival due to variable flow and climate (normalized by median)
  surv_adjustment <- calc_e2f_surv(q_cfs*flow_adjustment)/
    calc_e2f_surv(peak_q_median)

  # e2f multiplier is product of the two ratios
  det_recipr *surv_adjustment
  }


#' Convert annual air temperature into water temperature adjustment value
#'
#' @param x annual air temperature
#'
#' @return annual temperature to be added to modeled reach temperature
#'
#' @noRd

calc_waterT_diff <- function(air_temp) {

  water_t <- b0 + b1 * air_temp # calc water temp with linear equation

  water_t - (b0 + b1 * airT_mean) # calc difference between water temp and mean
}


#' Run the habitat model with stochastic params
#'
#' Use the MARSS simulation to generate annual variation in
#' temperature, wetted width and peak flow. This function calculates those
#' annual variations and brings them into the habitat model.
#'
#' Each param has a different way of being applied
#' q_low -- rearing capacity -- scalar * wetted width -- reach level
#' q_peak -- incubation -- scalar * egg_fry_surv -- on subbasin level params
#' t_peak -- rearing capacity and survival-- scalar + temperature -- reach level
#'
#' Note the output is automatically run through lcm_assign().
#'
#' @param pop which population to run
#' @param hab_scenario which scenario to run
#' @param era which era to run? curr, mid, late
#' @param beav_model which beaver model to run  ("BIP" or "SP")
#' @param runs How many runs/iterations? vector of values, e.g. 1:10
#' @param years How many years? Vector of values, e.g. 1:110 (default)
#' @param print_status Print model progress to console?
#'
#' @export
hab_run_stochastic <- function(pop,
                               hab_scenario,
                               era,
                               beav_model = "BIP",
                               runs = 1,
                               years = 1:100,
                               print_status = FALSE) {

  # calculate temp diff, width mult and e2f adj for years/runs
  df_marss_wide <- harp::df_marss %>%
    mutate(across(c(year, run), as.numeric)) %>%
    filter(year %in% years, run %in% runs) %>%
    tidytable::pivot_wider(names_from = var, values_from = value) %>%
    mutate(
      temp_diff_i = calc_waterT_diff(t_peak),
      lowQ_RI = calc_RI_low(q_low),
      e2f_i = dplyr::case_when( # need to update!
        pop == 'winter_steelhead' ~ calc_e2f_mult(q_peak_win_sthd, era),
        pop == 'summer_steelhead' ~ calc_e2f_mult(q_peak_sum_sthd, era),
        TRUE ~ calc_e2f_mult(q_peak, era))) %>%
    select(run, year, temp_diff_i, q_low, lowQ_RI, e2f_i)

  scenarios <- tidyr::crossing( # create all combinations of inputs
    pop,
    run_scenario = hab_scenario,
    run_era = era,
    df_marss_wide)

  purrr::pmap_dfr( # for each row, adjust flowlines and run habitat model
    scenarios,
    function(
    pop, run, year,
    run_scenario, run_era,
    q_low, lowQ_RI,
    e2f_i,
    temp_diff_i){
      # adjust temperatures
      temp_i <- harp::temperature_df %>%
        filter(era == run_era, scenario == run_scenario) %>%
        mutate(
        tidytable::across(c('temp_7DADM', 'temp_prespawn'), ~.x + temp_diff_i),
        temp_mdm = temp_Jn121 + hab_mwmt_to_mdm(temp_diff_i)) %>%
        select(-era)

      # adjust widths
      width_i <- width_df %>%
        filter(era == run_era) %>%
        mutate(RI = lowQ_RI, lowQ_i = q_low) %>%
        select(-era)

      h <- suppressMessages( # run hab model
        hab_run(
          pop,
          run_scenario,
          run_era,
          beav_model,
          temp_i,
          width_i))

      h_scaled <- dplyr::mutate( # adjust egg to fry survival
        h,
        value = ifelse(param == 'eggtofry_surv', value * e2f_i, value),
        run = as.numeric(run),
        year = year
        #year = as.numeric(year)
        )

      if(print_status) {message(paste('Run', run, '-- Year', year))}

      return(h_scaled)

    })
}
