
#' Assign habitat parameters
#'
#' Rearrange the habitat parameters into wide format and add lcm parameters (e.g. params_coho).
#' Species specific cleanup actions taken.
#'
#' @param hab_scenario_params dataframe output of hab_run()
#' @param chin_PSM should Chinook experience prespawn mortality? Default is TRUE
#' @export

lcm_assign <- function(hab_scenario_params, chin_PSM = TRUE){

  run_pop <- unique(hab_scenario_params$pop)

  if(run_pop %in% c("winter_steelhead", "summer_steelhead")){
    stop("Use lcm_assign_steelhead() for summer or winter steelhead runs")
  }

  # Create df of hab params and subbasin
  # move to habitat model
  dat_df <- dplyr::left_join(
    hab_scenario_params, subbasin_names, by = "subbasin_num") %>%
    dplyr::mutate(subbasin = as.factor(noaa_subba),
                  pop = run_pop) %>%
    dplyr::arrange(as.numeric(subbasin_num)) # sort by subb number

  # Convert to wide format
  # and append fixed variables from data-raw/params/params_pop.R
  dat_df <- dat_df %>%
    tidyr::pivot_wider(names_from = param, values_from = value) %>%
    append(get(paste0('params_', run_pop))) %>%
    data.frame() %>% # convert back to df
    replace(is.na(.), 0)  # remove NA and NaN

  # columns to group by if present
  group_vars <- c("scenario", "era", "run", "year")

  dat_df <- dat_df %>%
    dplyr::group_by(across(any_of(group_vars))) %>%
    dplyr::mutate(pop_init = pop_init * (eggs/sum(eggs))) %>%
    dplyr::ungroup() %>%
    dplyr::rename(
      egg_cap = eggs,
      egg_fry_surv = eggtofry_surv,
      S_up = prespawn_surv)

  if (run_pop == "coho") {
    # adjust values for coho
    dat_df <- dat_df %>%
      dplyr::mutate(movement = movement/100) %>%
      dplyr::rename(parr_surv = surv_s,
                    parr_cap = capacity_s,
                    parr_smolt_surv = surv_w,
                    parr_smolt_cap = capacity_w,
                    redist = movement)
  } #end if coho


  if (run_pop == "fall_chinook") {

    dat_df <- dat_df %>%
      dplyr::mutate(
        colnz_surv = surv_s^(1/9),# 1 week of freshwater mortality
        # Weekly productivity scaled with June temperatures
        # This is weighted so that 34% of fish will get the impact of warm June
        # temperatures for 3 of 8 weeks of the spring rearing period
        spring_surv = 0.66*surv_s^(8/9)+0.34*surv_s^(5/9)*surv_s_t^(3/9),
        # adjust hatchery smolt FW survival based on era/fork specific survivals
        # these are the ratio of no-action future June productivities to current
        fw_surv_h_nf = dplyr::case_match( # vectorized `switch`
          era,
          "curr" ~ 1 * fw_surv_h,
          "mid" ~ 0.999 * fw_surv_h,
          "late" ~ 0.994 * fw_surv_h),
        fw_surv_h_sf = dplyr::case_match(
          era,
          "curr" ~ 1 * fw_surv_h,
          "mid" ~ 0.999 * fw_surv_h,
          "late" ~ 0.988 * fw_surv_h)) %>%
      dplyr::select(-c(surv_s, surv_s_t, capacity_s_t, fw_surv_h)) %>%
      dplyr::rename(
        S_up_h = prespawn_h_surv,
        spring_cap = capacity_s,
        summer_cap = capacity_s_2,
        winter_surv = surv_w,
        summer_surv = surv_s_2)

    if(!chin_PSM){ # change calibrated parameters if not using Chinook PSM
      dat_df <- dat_df %>%
        dplyr::mutate(
          S_up = 1, S_up_h = 1, # full upstream survival
          subyr_ratio = params_chinook_noPSM["subyr_ratio"], # new params
          ps_parr_surv = params_chinook_noPSM["ps_parr_surv"],
          fry_migr_prod = params_chinook_noPSM["fry_migr_prod"],
          ps_yr_surv = params_chinook_noPSM["ps_yr_surv"]
        )
    }
  } # end if chinook

  if (run_pop == "steelhead") {

    dat_df <- dplyr::rename(
      dat_df,
      parr_surv = surv_s,
      parr_cap = capacity_s,
      first_winter_surv = surv_w,
      first_winter_cap = capacity_w,
      second_summer_cap  = capacity_s_2,
      second_summer_surv = surv_s_2,
      second_winter_cap  = capacity_w_2,
      second_winter_surv = surv_w_2
    )
  }

  if (run_pop == 'chum') {

    dat_df <- dplyr::rename(
      dat_df,
      fry_colonization_surv = fry_colonization_surv,
      fry_colonization_cap = fry_colonization_cap,
      fry_migrant_surv = surv_s
    )
  }

  dat_df

}
