#' Chinook LCM
#'
#' Define the life cycle model for chinook.
#'
#' @param mat_in N matrix
#' @param mat_out blank N matrix
#' @param hab_params param list
#' @param yr_label simulation year label
#'
#' @export
lcm_chinook <- function(mat_in, mat_out, hab_params, yr_label) {

  list2env(hab_params, envir = environment())

  females_nor <- diag(mat_in['spawners_nor',,]) * sex_ratio_nor
  females_hor <- diag(mat_in['spawners_hor',,]) * sex_ratio_hor
  females_total <- females_nor + females_hor

  # percent hatchery-origin females spawning in the wild
  phofs <- replace_na(females_hor/females_total, 0)

  eggs <- BH_func(S = females_total, p = fecund, c = egg_cap)

  pre_fry <- eggs * (1-phofs) * egg_fry_surv + # from nat-origin females
    eggs * phofs * egg_fry_surv * rrs_h # from hatch-origin females

  # store values
  diag(mat_out['eggs',, ]) <- eggs
  diag(mat_out['pre_fry',, ]) <- pre_fry

  # subyearlings that migrate after 12 weeks
  # week #1
  natal_fry <- BH_func(S = pre_fry, p = colnz_surv, c = spring_cap*2)

  # hatchery release
  hatchery_release <-  # Total run (hatchery-origin)
    nf_release * hatchery_table$nf_out  + # NF releases
    sf_release * hatchery_table$sf_out  # SF releases

  # natural-origin weeks #2-9
  natal_fry_distrib <- distrib_fish(natal_fry, mat_move_downstream, 1)
  natal_fry_distrib_vec <- colSums(natal_fry_distrib)
  parr_distrib_DI <- natal_fry_distrib_vec*spring_surv

  # calculate percent natural-origin juveniles in rearing basins
  nat_perc <- parr_distrib_DI / (parr_distrib_DI + hatchery_release)

  #B-H capacity gets modified by nat_perc to simulate competition
  parr <- BH_func_distrib( # these are conditions in the rearing basin
    natal_fry_distrib, spring_surv, spring_cap*2*nat_perc)
  parr_migrants <- parr * subyr_ratio

  # subyearlings that migrate after 1 week
  fry_migrants <- ((pre_fry * colnz_surv) - natal_fry)*fry_migr_prod

  # fish that attempt to become yearlings
  stayers <- parr * (1-subyr_ratio) # store matrix
  summer_survivors <- BH_func_distrib(
    stayers, summer_surv, summer_cap)

  # store values
  diag(mat_out['natal_fry',, ]) <- natal_fry
  diag(mat_out['natal_fry_distrib',, ]) <- natal_fry_distrib_vec
  mat_out['parr',, ] <- parr
  mat_out['parr_migrants',, ] <- parr_migrants
  mat_out['stayers',,] <- stayers
  mat_out['summer_survivors',,] <- summer_survivors
  diag(mat_out['fry_migrants',, ]) <- fry_migrants

  # YEAR 2 ---------------------------------------------------------------------
  # bring in survivors from previous year
  # then multiply by the winter survival of the REARING subbasin
  # yearlings outmigrate in the spring

  yearlings <- t(t(mat_in['summer_survivors',,]) * winter_surv)

  mat_out['yearlings',,] <- yearlings

  # ESTUARY --------------------------------------------------------------------
  # Apply bay survival (ds migration, delta, bay, nearshore productivity)

  # split fry between delta and nearshore
  # combine all fry in delta, then reassign the survivors (proportional to input)
  delta_fry <- BH_func(sum(fry_migrants), delta_prod[1], capacity_delta[1])*
    prop.table(fry_migrants)
  diag(mat_out['delta_fry',,]) <- delta_fry
  diag(mat_out['nearshore_fry',,]) <- fry_migrants - delta_fry

  # Ocean survival, subyearlings
  diag(mat_out['ocean0_0',, ]) <- (delta_fry + rowSums(parr_migrants)) * ps_parr_surv
  mat_out['ocean0_1',, ] <- mat_in['ocean0_0',, ] * so1 # 2 year olds, ocean1
  mat_out['ocean0_2',, ] <- mat_in['ocean0_1',, ] * (1 - b0_2) * so2
  mat_out['ocean0_3',, ] <- mat_in['ocean0_2',, ] * (1 - b0_3) * so3
  mat_out['ocean0_4',, ] <- mat_in['ocean0_3',, ] * (1 - b0_4) * so4

  # ocean survival, yearlings
  diag(mat_out['ocean1_0',, ]) <- rowSums(yearlings) * ps_yr_surv
  mat_out['ocean1_1',, ] <- mat_in['ocean1_0',, ] * so1
  mat_out['ocean1_2',, ] <- mat_in['ocean1_1',, ] * (1 - b1_2) * so2
  mat_out['ocean1_3',, ] <- mat_in['ocean1_2',, ] * (1 - b1_3) * so3

  mat_out['total_run_subyr',,] <- #Total run (subyearling migrants, natural)
    b0_2 * mat_in['ocean0_1',, ] +
    b0_3 * mat_in['ocean0_2',, ] +
    b0_4 * mat_in['ocean0_3',, ] +
    mat_in['ocean0_4',,]

  mat_out['total_run_yr',,] <- #Total run (yearling migrants)
    b1_2 * mat_in['ocean1_1',, ] +
    b1_3 * mat_in['ocean1_2',, ] +
    mat_in['ocean1_3',, ]

  mat_out['total_run_nor',,] <-
    mat_out['total_run_subyr',,] +
    mat_out['total_run_yr',,]

  total_run_hor <-  # Total run (hatchery-origin)
    nf_release * hatchery_table$nf_dist * fw_surv_h_nf * sw_surv_h + # NF returners
    sf_release * hatchery_table$sf_dist * fw_surv_h_sf * sw_surv_h # SF returners

  diag(mat_out['total_run_hor',,]) <- total_run_hor

  mat_out['total_run',,] <-
    mat_out['total_run_nor',,] +
    mat_out['total_run_hor',,]

  # collect broodstock proportionally from subbasins based on run size
  # move 140 to raw-data later
  # move year into hab params to improve warning message

  total_run_nor <- diag(mat_out['total_run_nor',,])

  # Target broodstock by origin. tot_bs is split evenly between HOR and NOR
  # and then split proportionally by the number of spawners.
  # This simulates random fishing for two separate NOR and HOR targets as fish
  # pass through the lower NF
  bs_target_NOR <- replace_na(
    prop.table(total_run_nor * bs_subbasins) * tot_bs/2,
    0) # replace NAs with 0 when there's no run
  bs_target_HOR <- replace_na(
    prop.table(total_run_hor * bs_subbasins) * tot_bs/2,
    0)

  # actual broodstock - the target number, split by subbasin or however many
  # spawners are in the subbasin - whichever is lower
  broodstock_NOR <- pmin(bs_target_NOR, total_run_nor)
  broodstock_HOR <- pmin(bs_target_HOR, total_run_hor)


  # NOR spawners
  diag(mat_out['prespawn_nor',,]) <- total_run_nor - broodstock_NOR

  diag(mat_out['spawners_nor',,]) <-
    diag(mat_out['prespawn_nor',,]) * S_up * (1 - hr)

  diag(mat_out['spawners_hor',,]) <- # HOR spawners
    (total_run_hor - broodstock_HOR) *
    S_up_h * (1 - hr) # separate prespawn survival for HOR

  mat_out['spawners',,] <- mat_out['spawners_nor',,] + mat_out['spawners_hor',,]

  return(mat_out)

} # end chinook
