#' Replace NA with mean of group
#'
#' @param x a numeric vector
impute_mean <- function(x) replace(x, is.na(x), mean(x, na.rm = TRUE))

#' Break slope into classes "low", "medium" and "high"
#'
#' @param x a numeric vector
reclass_slope <- function(x){
  cut(x,
      breaks= c(-Inf, 0.02, 0.04, Inf),
      labels = c("low", "med", "high"),
      right = FALSE) %>% # slope of 0.2 is medium
    as.character.factor()
}

#' Convert square meters to hectares
#'
#' @param x area in m
m2_to_ha <- function(x){
  x / 10000
}

#' Join some flowline attributes onto other habitats
#'
#' @param x a habitat data frame
#' @param df_flowline a species-specific flowline dataset
join_flowline <- function(x, df_flowline){
  df_flowline %>%
    select(tidyr::any_of(c(
      "noaaid",
      "noaa_subba",
      "subbasin_num",
      "length",
      "lc",
      "slope",
      "pop",
      "spawn", "rear"))) %>%
    tidytable::right_join(x, by = "noaaid")}

#' Adjust summer wetted width based on climate
#'
#' @param width current median wetted wdith
#' @param RI low flow recurrence interval
#' @param lowQ_i stochastic flow or median flow for deterministic model
#' @param a a constant
#' @param b a constant
adjust_summer_width <- function(width, RI, lowQ_i, a, b){
  cc_const <- 1+a*RI^b # climate change component
  geom_const <- 0.5 # this is "b" in the at-a-station hydraulic geom eq.
  width * (lowQ_i*cc_const/low_q_median)^geom_const
}

#' Convert flow at Arlington gage to egg to basinwide incubation survival
#'
#' Equation derived in 2021 Stillaguamish Smolt Trap Report
#'
#' @param q_cfs peak flow, cfs
calc_e2f_surv <- function(q_cfs){30.154*exp(q_cfs*-7*10^-5)}

#' Format input parameter vector
#'
#' @param in_vec vector to format
format_vec <- function(in_vec){
  paste(sprintf('`"%s"`', in_vec), collapse = ", ")
}
