# calibrate SAR and age structure for steelhead
# devtools::load_all(export_all = F)
library(tidyverse)
library(harp) #check that installation is up to date first!
library(furrr)
plan(multisession, workers = 16)

vers <- "v0.4.0" # update manually
nsims <- 100

out_dir <-
  "//nwcfile.nmfs.local/FE/watershed/Stilly Phase 2 Shared"

fig_dir <- file.path(out_dir, "Figures/cal_diagnostic")
hab_dir <- file.path(
  out_dir, "lcm_results", vers, "single_scenario", nsims)

basin <- "STL"

# current (No action) total delta capacity
dcap_current <- harp:::params_hab_fall_chinook$delta_cap_curr
dcap_hist <- harp:::params_hab_fall_chinook$delta_cap_hist

chinook_base <- hab_run("fall_chinook", "Current", "curr") %>% lcm_assign()

# Calibration targets ----------------------------------------------------------
fry_ratio_target <- 30.4/69.6 # ratio from smolt trap
fry_vector <- c(0.304, 0.696)
SAR_target <- 0.0038  # target SAR
SAR_target_yr <- 0.0058
subyearling_target <- c(0.99, 0.01) # yearlings avg 1%
nRunGlobal <- 2000
nRunLocal <- 2000
bs_target <- 70

# fry ratio targets (calibrate to each of these, separately)
dcap_targets <- seq(0.2, 1, 0.2)

# function to calculate modified RMSE based on:
# x: a vector of "guesses" for productivities or maturation rates
# dcap_perc: a target percentage representing the percent of delta residents
#   originating in the natal basin
calc_error_score <- function(x, dcap_perc){
  all_lcm <- chinook_base %>%
    mutate( # run LCM using values from input vector
      subyr_ratio = x[1],
      ps_parr_surv = x[2],
      fry_migr_prod = x[3],
      ps_yr_surv = x[4],
      capacity_delta = dcap_current * dcap_perc) %>% # modify delta cap.
    lcm_run("all") %>%
    filter(year == 100)

  fry_ratio <- all_lcm %>%
    filter(lifestage %in% c("parr_migrants", "fry_migrants")) %>%
    group_by(lifestage) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    cbind(target = fry_ratio_target) %>%
    mutate(
      fry_ratio = fry_migrants/parr_migrants,
      fry_diff = ((fry_ratio - target)/100)^2) %>%
    pull(fry_diff)

  subyr_SAR <- all_lcm %>%
    filter(
      lifestage %in% c(
        "parr_migrants", "fry_migrants",
        "spawners_nor",
        'total_run_nor', 'total_run_subyr')) %>%
    group_by(lifestage) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    transmute(
      smolt = parr_migrants+fry_migrants,
      # only include N-O adults that outmigrated as subyearlings
      adult = spawners_nor * (total_run_subyr/total_run_nor)) %>%
    cbind(target = c(SAR_target)) %>%
    mutate(
      SAR = (adult+bs_target)/smolt,
      SAR_diff = (SAR - target)^2) %>%
    pull(SAR_diff)

  subyr <- all_lcm %>%
    filter(
      lifestage %in% c('total_run_yr', 'total_run_subyr')) %>%
    mutate(
      fw_age = stringr::str_extract(lifestage, "(?<=total_run_)\\w+")) %>%
    group_by(fw_age) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    ungroup() %>%
    mutate(perc = n/sum(n)) %>%
    filter(fw_age == "yr") %>% # match yearlings (matching both is redundant)
    cbind(target = subyearling_target[2])%>%
    mutate(sq_diff = ((perc-target)/10)^2) %>%
    pull(sq_diff)

  yr_SAR <- all_lcm %>%
    filter(
      lifestage %in% c(
        "yearlings", "spawners_nor",
        'total_run_nor', 'total_run_yr')) %>%
    group_by(lifestage) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    transmute(
      smolt = yearlings,
      # only include N-O adults that outmigrated as yearlings
      adult = spawners_nor * (total_run_yr/total_run_nor)) %>%
    cbind(target = c(SAR_target_yr)) %>%
    mutate(
      SAR = adult/smolt,
      SAR_diff = (SAR - target)^2) %>%
    pull(SAR_diff)

  #weight SARs 2x other targets because values are so small
  sqrt(sum(fry_ratio, subyr_SAR*2, yr_SAR*2, subyr)/6)
}

calibrate_each <- function(dcap_perc){
  # nloptr will try to minimize the value of the function you give it.
  # Here, it will try to minimize the score from the function calc_error_score
  nlopt1 <- nloptr::nloptr( # run nonlinear optimizer
    x0 = c(0.99,
           0.03,
           0.04,
           0.01),
    eval_f = calc_error_score,
    lb = rep(0,4),
    ub = rep(1,4),
    dcap_perc = dcap_perc,
    opts = list(
      algorithm ="NLOPT_GN_DIRECT_L", # global optimization algorithm
      # print_level = 1,
      maxeval = nRunGlobal))

  results1 <- nlopt1$solution # store results of global optimization

  nlopt2 <- nloptr::nloptr( # run nonlinear optimizer
    x0 = results1, # start with global results
    eval_f = calc_error_score,
    lb = c(0.8, 0.010, 0.0, 0.019),
    ub = c(1.0, 0.025, 0.2, 0.020),
    dcap_perc = dcap_perc,
    opts = list(
      algorithm ="NLOPT_LN_COBYLA", # local optimization algorithm
      # print_level = 1,
      maxeval = nRunLocal))

  results2 <- setNames(
    nlopt2$solution,
    c("syr", "psps", "fmp", "psys")) # name cols for rbind later

  c(results2, dcap_perc = dcap_perc) # output the best parameter vector
}

cal_out <- future_map_dfr(dcap_targets, calibrate_each) # calibrate to each target

saveRDS(cal_out, "ignore/calparams_vs_deltacomp_diagnostic.RDS")

# make two different restoration senarios
chin_NoAc <-
  list.files(hab_dir, pattern = "Current_curr", full.names = TRUE) %>%
  readRDS() %>%
  lcm_assign()


# function to apply calibration to given scenario
apply_cal <- function(params, restored){
  chin_NoAc %>%
    mutate(
      subyr_ratio = params$syr,
      ps_parr_surv = params$psps,
      fry_migr_prod = params$fmp,
      capacity_delta = ifelse(
        restored,
        dcap_hist*params$dcap_perc,
        dcap_current*params$dcap_perc)
      # capacity_delta = case_when(
      #   nonnatal_treatment == "fixed_percent" ~
      #     capacity_delta*params$dcap_perc,
      #   nonnatal_treatment == "fixed_number" ~
      #     capacity_delta-(dcap_current*(1-params$dcap_perc)))
      ) %>%
    lcm_run("spawners_nor") %>%
    summarize(n = sum(n), .by = c(year, run)) %>%
    summarize(n = median(n)) %>%
    pull(n)}


apply_each_cal <- function(sc_name){
  restored <- sc_name == "Delta"
  future_map_dfr(
    split(cal_out, cal_out$dcap_perc), # split data fram by cap %
    # could also just use map instead of map_dfr in calibration
    ~c(spawners = apply_cal(.x, restored), #
       dcap_perc = .x$dcap_perc,
       scenario = sc_name))}

spawners <- map_dfr(c("No Action", "Delta"), apply_each_cal) %>%
  mutate(
    spawners = as.numeric(spawners),
    dcap_perc = as.numeric(dcap_perc))

saveRDS(spawners, "ignore/spawners_vs_deltacomp_diagnostic.RDS")
# # plot returners
# spawners %>%
#   filter(nonnatal_treatment == "fixed_percent") %>%
#   # mutate(
#   #   pretty_facets = factor(
#   #     nonnatal_treatment,
#   #     levels = c("fixed_number", "fixed_percent"),
#   #     labels = c(
#   #       "Restoration Scenario\nHas Equal #\nof Nonnatal Fry",
#   #       "Restoration Scenario\nHas Equal %\nof Nonnatal Fry"))) %>%
#   ggplot()+
#   geom_point(
#     aes(dcap_perc, spawners, color = scenario))+
#   geom_line(
#     linetype = 2,
#     aes(dcap_perc, spawners, color = scenario, group = scenario))+
#   scale_x_continuous(
#     name = "Current Percent Natal Fry",
#     labels = scales::percent)+
#   scale_y_continuous(
#     name = "Natural-Origin Spawners",
#     labels = scales::comma)+
#   scale_color_discrete(name = "Scenario")+
#   # facet_grid(~pretty_facets)+
#   theme_bw()
#
# ggsave(
#   file.path(
#     out_dir,
#     sprintf("delta_comp_Neq_%s_%s.png", basin, Sys.Date())),
#   width = 6, height = 4)
#
# spawners %>%
#   group_by(dcap_perc, nonnatal_treatment) %>%
#   mutate(
#     n_curr = spawners[scenario == 'No Action'],
#     n_change = spawners - n_curr,
#     perc_change = n_change/n_curr,
#     pretty_facets = factor(
#       nonnatal_treatment,
#       levels = c("fixed_number", "fixed_percent"),
#       labels = c(
#         "Restoration Scenario\nHas Equal #\nof Nonnatal Fry",
#         "Restoration Scenario\nHas Equal %\nof Nonnatal Fry"))) %>%
#   ggplot()+
#   geom_point(
#     aes(dcap_perc, perc_change, color = scenario))+
#   geom_line(
#     linetype = 2,
#     aes(dcap_perc, perc_change, color = scenario, group = scenario))+
#   scale_x_continuous(
#     name = "Current Percent Natal Fry",
#     labels = scales::percent)+
#   scale_y_continuous(
#     name = "Percent Change from Current",
#     labels = scales::percent)+
#   scale_color_discrete(name = "Scenario")+
#   facet_grid(~pretty_facets)+
#   theme_bw()
#
# ggsave(
#   file.path(
#     out_dir,
#     sprintf("delta_comp_perc_change_%s_%s.png", basin, Sys.Date())),
#   width = 6, height = 4)
