# calibrate SAR and age structure for steelhead
#devtools::load_all(export_all = F)
library(harp) # make sure installed version is up to date first!!
library(tidyverse)

vers <- "v0.4.0" # update manually
nsims <- 100

out_dir <-
  "//nwcfile.nmfs.local/FE/watershed/Stilly Phase 2 Shared"

fig_dir <- file.path(out_dir, "Figures/cal_diagnostic")
hab_dir <- file.path(
  out_dir, "lcm_results", vers, "single_scenario", nsims)

basin <- "STL"

chinook_base <- hab_run("fall_chinook", "Current", "curr") %>% lcm_assign()

# calibration targets
SAR_target <- 0.0038  # target SAR
SAR_target_yr <- 0.0058
subyearling_target <- c(0.99, 0.01) # yearlings avg 1%
nRunGlobal <- 2000
nRunLocal <- 2000
bs_target <- 70

# fry ratio targets (calibrate to each of these, separately)
fr_targets <- seq(0.2, 0.8, 0.1)

# function to calculate modified RMSE based on:
# x: a vector of "guesses" for productivities or maturation rates
# fry_target: a target percentage representing the percent of subyearling
#   outmigrants that are fry-sized
calc_error_score <- function(x, fry_target){
  all_lcm <- chinook_base %>%
    mutate( # run LCM using values from input vector
      subyr_ratio = x[1],
      ps_parr_surv = x[2],
      fry_migr_prod = x[3],
      ps_yr_surv = x[4]) %>%
    lcm_run("all") %>%
    filter(year == 100)

  # error due to mismatched fry ratio:
  fry_ratio <- all_lcm %>%
    filter(lifestage %in% c("parr_migrants", "fry_migrants")) %>%
    group_by(lifestage) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    cbind(target = fry_target) %>% # fry_target is a function param
    mutate(
      fry_ratio = fry_migrants/(parr_migrants+fry_migrants),
      fry_diff = ((fry_ratio - target)/10)^2) %>%
    pull(fry_diff)

  # error due to mismatched subyearling SAR
  subyr_SAR <- all_lcm %>%
    filter(
      lifestage %in% c(
        "parr_migrants", "fry_migrants",
        "spawners_nor",
        'total_run_nor', 'total_run_subyr')) %>%
    group_by(lifestage) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    transmute(
      smolt = parr_migrants+fry_migrants,
      # only include N-O adults that outmigrated as subyearlings
      adult = spawners_nor * (total_run_subyr/total_run_nor)) %>%
    cbind(target = c(SAR_target)) %>%
    mutate(
      SAR = (adult+bs_target)/smolt,
      SAR_diff = (SAR - target)^2) %>%
    pull(SAR_diff)

  # error due to mismatched yearling SAR
  yr_SAR <- all_lcm %>%
    filter(
      lifestage %in% c(
        "yearlings", "spawners_nor",
        'total_run_nor', 'total_run_yr')) %>%
    group_by(lifestage) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    pivot_wider(names_from = lifestage, values_from = n) %>%
    transmute(
      smolt = yearlings,
      # only include N-O adults that outmigrated as yearlings
      adult = spawners_nor * (total_run_yr/total_run_nor)) %>%
    cbind(target = c(SAR_target_yr)) %>%
    mutate(
      SAR = adult/smolt,
      SAR_diff = (SAR - target)^2) %>%
    pull(SAR_diff)

  # error due to mismatched subyearling/yearling composition
  subyr <- all_lcm %>%
    filter(
      lifestage %in% c('total_run_yr', 'total_run_subyr')) %>%
    mutate(
      fw_age = stringr::str_extract(lifestage, "(?<=total_run_)\\w+"),
    ) %>%
    group_by(fw_age) %>%
    summarize(n = sum(n), .groups = "keep") %>%
    ungroup() %>%
    mutate(perc = n/sum(n)) %>%
    filter(fw_age == "yr") %>% # match yearlings (matching both is redundant)
    cbind(target = subyearling_target[2])%>%
    mutate(sq_diff = ((perc-target)/10)^2) %>%
    pull(sq_diff)

  #weight SARs 2x other targets because values are so small
  sqrt(sum(fry_ratio, subyr_SAR*2, yr_SAR*2, subyr)/6)
}

# function to run a full model calibration, given a fry target
calibrate_each <- function(fry_target){
  # nloptr will try to minimize the value of the function you give it.
  # Here, it will try to minimize the score from the function calc_error_score
  nlopt1 <- nloptr::nloptr( # run nonlinear optimizer
    x0 = c(0.94, # x0 is the vector of starting values.
           0.02,
           0.12,
           0.02),
    eval_f = calc_error_score,
    lb = rep(0,4),
    ub = rep(1,4),
    fry_target = fry_target,
    opts = list(
      algorithm ="NLOPT_GN_DIRECT_L", # global optimization algorithm
      #print_level = 1,
      maxeval = nRunGlobal))

  results1 <- nlopt1$solution # store results of global optimization

  nlopt2 <- nloptr::nloptr( # run nonlinear optimizer
    x0 = results1, # start with global results
    eval_f = calc_error_score,
    lb = rep(0,4),
    ub = rep(1,4),
    fry_target = fry_target,
    opts = list(
      algorithm ="NLOPT_LN_COBYLA", # local optimization algorithm
      #print_level = 1,
      maxeval = nRunLocal))

  results2 <- setNames(
    nlopt2$solution,
    c("syr", "psps", "fmp", "psys")) # name cols for rbind later

  c(results2, fry_target = fry_target) # output the best parameter vector
}

cal_out <- future_map_dfr(fr_targets, calibrate_each) # calibrate to each value

saveRDS(
  cal_out,
  sprintf("ignore/calparams_vs_fryratio_diagnostic_%s.RDS", Sys.Date()))

# make four different restoration scenarios
chin_curr <-
  list.files(hab_dir, pattern = "Current_curr", full.names = TRUE) %>%
  readRDS() %>%
  lcm_assign()


# function to apply calibration and run a lifecycle model, given habitat data
apply_cal <- function(params){
  chin_curr %>%
    mutate(
      subyr_ratio = params$syr,
      ps_parr_surv = params$psps,
      fry_migr_prod = params$fmp,
      ps_yr_surv = params$psys) %>%
    lcm_run("spawners_nor")  %>%
    summarize(n = sum(n), .by = c(year, run)) %>%
    summarize(n = median(n)) %>%
    pull(n)}

cal_out %>%
  dplyr::mutate(dplyr::across(.cols = everything(), ~signif(.x, 2)))


spawners <- future_map_dfr(
  split(cal_out, cal_out$fry_target), # split data fram by cap %
  # could also just use map instead of map_dfr in calibration
  ~c(spawners = apply_cal(.x), #
     fry_target = .x$fry_target))%>%
  mutate(
    spawners = as.numeric(spawners),
    fry_target = as.numeric(fry_target))

saveRDS(
  spawners,
  sprintf("ignore/spawners_vs_fryratio_diagnostic_%s.RDS", Sys.Date()))

# plot returners
spawners %>%
  ggplot()+
  geom_point(
    aes(fry_target, spawners))+
  geom_line(
    linetype = 2,
    aes(fry_target, spawners))+
  scale_x_continuous(
    name = "Current Estimated % Fry-sized Subyearlings",
    labels = scales::label_percent())+
  scale_y_continuous(
    name = "Natural-Origin Spawners",
    labels = scales::comma)+
  scale_color_discrete(name = "Scenario")+
  theme_bw()

ggsave(
  file.path(
    fig_dir,
    sprintf("delta_fryratio_Neq_%s_%s.png", basin, Sys.Date())),
  width = 6, height = 4)
