# Script to run scenarios for diagnostics with full LCM outputs

reinstall_harp <- function(){
  devtools::uninstall()
  devtools::install(dependencies = FALSE)
}


# Git status checks ------------------------------------------------------------

# reinstall_harp() # run this if needed to reinstall package
git_status <- harp:::check_git_status()
dir_name <- git_status["dir_name"]
cur_branch <- git_status["cur_branch"]
cur_tag <- git_status["cur_tag"]


## Prep data directories -------------------------------------------------------

# share drive directory
path_to_shared_folder <-
  '//nwcfile.nmfs.local/FE/watershed/Stilly Phase 2 Shared/lcm_results'

if (cur_branch == 'main' & !dir.exists(path_to_shared_folder)) {
  warning('Not connected to the shared drive. Results will not be archived')
}

if(grepl("dirty", dir_name)){
  warning('the repo is dirty. Results will not be archived')}

if(cur_branch == 'main' & dir.exists(path_to_shared_folder)){
  out_path <- file.path(path_to_shared_folder, dir_name)
} else{
  out_path <- paste0('ignore/results/', dir_name)
}

sens_path <- file.path(out_path, "sensitivity")

try(dir.create(sens_path, recursive = TRUE), silent == T)

message(sprintf("Saving results to %s", out_path))

# Packages and parallel setup --------------------------------------------------
library(harp)
# library(furrr)
library(tidyverse)
library(tidytable)
# library(progressr)
# plan(multisession, workers = 16)
# handlers(global = TRUE) # show progress bars

# Lifecycle model --------------------------------------------------------------

read_hab <- function(hab_path){ # function to read one habitat data file
  readRDS(hab_path)
}


# server_dir <-
#   "//nwcfile.nmfs.local/FE/watershed/Stilly Phase 2 Sharedfigures/sensitivity"
#
# temp_save_dir <- "ignore/sensitivity"

# lifestages and groupings
# table to recode parameters
code_table <- tribble( # just chinook params for no-PSM model
  ~pop,           ~param,               ~lifestage,       ~type,          ~grp,
  "fall_chinook", "egg_cap",            "Spawning",       "Capacity",     "sp",
  "fall_chinook", "egg_fry_surv",       "Incubation",     "Productivity", "sp",
  "fall_chinook", "spring_cap",         "Subyearling",    "Capacity",     "s",
  "fall_chinook", "colnz_surv",         "Colonization",   "Productivity", "s",
  "fall_chinook", "spring_surv",        "Subyearling",    "Productivity", "s",
  "fall_chinook", "summer_cap",         "Summer",         "Capacity",     "s2",
  "fall_chinook", "summer_surv",        "Summer",         "Productivity", "s2",
  "fall_chinook", "capacity_w",         "Winter",         "Capacity",     "w",
  "fall_chinook", "winter_surv",        "Winter",         "Productivity", "w",
  #"fall_chinook", "capacity_delta",     "Delta",          "Capacity",     "d",
  # "fall_chinook", "S_up",               "Prespawn (N-O)",   "Productivity", "sp",
  # "fall_chinook", "S_up_h",             "Prespawn (H-O)",   "Productivity", "sp",
)

# function to get capacity-weighted averages from lcm_assign output
weight_basin <- function(x){ # x is input table
  hab_cols <- 13:ncol(x)
  x %>%
    pivot_longer( # make long
      all_of(hab_cols),
      names_to = "param",
      values_to = "value") %>%
    left_join(code_table, by = c("pop", "param")) %>%  # class lifestages
    filter(!is.na(lifestage)) %>%
    mutate(
      weights = value[type == "Capacity"], # get capacities
      value = case_when( # transform weekly values back to seasonal ones
        param == "colnz_surv" ~ value^8,
        param == "capacity_delta" ~ value/n(),
        TRUE ~ value),
      .by = c(pop, subbasin_num, grp)
    ) %>%
    group_by(pop, param, lifestage, type) %>%
    summarize(
      w_mean = weighted.mean(value, weights), # weighted mean
      basin_sum = sum(value), # basin sum
      .groups = "keep") %>%
    mutate(stat = ifelse(type == "Productivity", w_mean, basin_sum)) %>%
    select(pop, param, lifestage, type, stat)
}

# function to get current and historic habitat data
get_hab <- function(pop){
  # run habitat models ---------------------------------------------------------
  # get current era habitat
  h_curr_assign <- hab_run(pop, 'Current', "curr") %>% # run current conditions
    lcm_assign(chin_PSM = FALSE)
  h_hist_assign  <- hab_run(pop, 'Historical', "hist")%>% # run historic
    lcm_assign(chin_PSM = FALSE)

  # get subbasin-scale differences between eras --------------------------------
  hab_cols <- 13:ncol(h_curr_assign) # how many habitat cols are there?

  h_curr <- h_curr_assign %>% # format current hab data for lcm input
    pivot_longer(
      all_of(hab_cols),
      names_to = "param",
      values_to = "value_curr")

  h_hist <- h_hist_assign %>% # and historic data
    pivot_longer(
      all_of(hab_cols),
      names_to = "param",
      values_to = "value_hist") %>%
    select(subbasin_num, param, value_hist)

  # compare current and historic, return this df
  subbasin_diffs <- inner_join(
    h_curr, h_hist, by = c("subbasin_num", "param")) %>%
    ungroup() %>%
    mutate(vdiff = value_hist-value_curr)

  # get basin-scale differences ------------------------------------------------
  h_curr_basin <- weight_basin(h_curr_assign)
  h_hist_basin <- weight_basin(h_hist_assign)

  basin_diffs <- inner_join(
    h_curr_basin, h_hist_basin,
    by = c("pop", "param", "lifestage", "type")) %>%
    rename(v_curr = "stat.x", v_hist = "stat.y") %>%
    ungroup() %>%
    mutate(rest_potential = (v_hist - v_curr)/v_curr)

  list(
    subba = subbasin_diffs,
    basin = basin_diffs
  )}

# generic function to run sensitivity analysis with habitat input
run_generic <- function(hab_data, pop, mparam, adj){
  cat(sprintf("Running %s at %.0f%% %s\n", pop, adj*100, mparam))
  # clean up lcm input table
  lcm_input <- hab_data %>%
    mutate( # select value to change and change it
      value = ifelse(param == mparam, value_curr + (vdiff * adj), value_curr)) %>%
    select(-c(value_hist, value_curr, vdiff)) %>% # clean up
    pivot_wider(names_from = param, values_from = value) # make wide again

  target_lifestage <- ifelse(
    unique(hab_data$pop != "fall_chinook"), "spawners", "spawners_nor")

  # calculate spawner abundance
  lcm_run(lcm_input, target_lifestage) %>%
    filter(year == 100) %>% # change for stochastic version
    summarize(n = sum(n)) %>%
    mutate(
      pop = pop,
      param = mparam,
      adj = adj)
}

# function to run simulation for one species
run_simulation <- function(hab_data){ # takes a list of hab data
  # find out which params change from current to historic for this pop
  mutable_params <- hab_data[["basin"]] %>%
    transmute(pop, mparam = param)

  # table of what to vary and by how much
  adjustment_table <- expand_grid(mutable_params, adj = seq(0, 1, 0.1))

  # function to run lcm with single adjusted parameter
  run_specific <- function(pop, mparam, adj){
    run_generic(hab_data[["subba"]], pop, mparam, adj) %>%
      left_join(hab_data[["basin"]], by = c("pop", "param")) %>%
      mutate(percent_change = rest_potential*adj)
  }

  purrr::pmap_dfr(adjustment_table, run_specific) # function uses named parameters
}

# function to put everything together
run_one_pop <- function(pop){
  get_hab(pop) %>%
    run_simulation()
}

# run all of the pops
# all_runs <- map_dfr(c("coho", "steelhead", "fall_chinook"), run_one_pop)
all_runs <- run_one_pop("fall_chinook")
# save results ----------------------------------------------------------------

df_plt <- all_runs %>% # prepare data frame for plotting
  mutate(pop = factor(
    pop,
    levels = c("coho", "fall_chinook", "steelhead"),
    labels = c("Coho", "Chinook", "Steelhead"))) %>%
  filter(!(lifestage == "Winter" & pop == "Chinook" & type == "Capacity")) %>%
  na.omit()

df_endpoints <- df_plt %>%
  filter(adj == 1) %>%
  mutate(lab = lifestage) %>%
  na.omit() %>%
  mutate(scolor = ifelse(type == "Capacity", "#990c94", "#f287ee"))


saveRDS(df_endpoints,file.path(sens_path, "df_endpts_noPSM.RDS"))
saveRDS(df_plt,file.path(sens_path, "df_plt_noPSM.RDS"))

# plot everything -------------------------------------------------------------
watermark <-  annotate(
  "text", x = Inf, y = -Inf, label = cur_tag,
  hjust=1, vjust=-1.1,  col="black",
  fontface = "italic",
  alpha = 0.2)

# sens_plot <- df_plt %>%
#   ggplot(aes(
#     percent_change,
#     n,
#     color = type,
#     group = param
#   )) +
#   facet_wrap(pop~., scales = 'free', nrow = 3) +
#   cowplot::theme_half_open() +
#   geom_line(linewidth = 0.8) +
#   geom_point(data = df_endpoints) +
#   ggrepel::geom_label_repel(
#     data = df_endpoints,
#     aes(label = lab, segment.color = scolor),
#     nudge_x = Inf,
#     direction = "y",
#     hjust = 0.7, # vertical position is sensitive to hjust, unclear why
#     size = 3.2, # shrink labels to fit
#     segment.size = 0.3,
#     segment.linetype = 2,
#     segment.curvature = -0.3,
#     segment.inflect = TRUE,
#     show.legend = FALSE)+
#   scale_x_continuous(
#     expand = expansion(mult = c(0.05, 0.6)), # expand x axis to fit labels
#     labels = scales::label_percent()) +
#   scale_color_manual(
#     values = c(Capacity = "#990c94", Productivity = "#f287ee"))+
#   scale_y_continuous(labels = scales::label_comma()) +
#   labs(y = 'Spawners',
#        x = 'Restoration benefit',
#        color = NULL, title = "Stillaguamish") +
#   theme(legend.position = c(0, 0.98),
#         plot.background = element_rect(fill = "white"),
#         panel.spacing.y = unit(-0.02, "inches"),
#         panel.grid.major = element_line('grey90'),
#         plot.title = element_text(size = 13, face = "plain", hjust = 0.5),
#         strip.background = element_rect(fill = NA))+
#   watermark
#
# ggsave(
#   file.path(sens_path, sprintf("all_sensitivity_STL_%s.png", Sys.Date())),
#   width = 4,
#   height = 12)

chn_plot <- df_plt %>%
  filter(pop == "Chinook") %>%
  ggplot(aes(
    percent_change,
    n,
    color = type,
    group = param
  )) +
  cowplot::theme_half_open() +
  geom_line(linewidth = 0.8) +
  geom_point(data = filter(df_endpoints, pop == "Chinook")) +
  ggrepel::geom_label_repel(
    data = filter(df_endpoints, pop == "Chinook"),
    aes(label = lab, segment.color = scolor),
    nudge_x = Inf,
    direction = "y",
    hjust = 0.7, # vertical position is sensitive to hjust, unclear why
    size = 3.2, # shrink labels to fit
    segment.size = 0.3,
    segment.linetype = 2,
    segment.curvature = -0.3,
    segment.inflect = TRUE,
    show.legend = FALSE)+
  scale_x_continuous(
    expand = expansion(mult = c(0.05, 0.7)), # expand x axis to fit labels
    labels = scales::label_percent()) +
  scale_color_manual(
    values = c(Capacity = "#990c94", Productivity = "#f287ee"))+
  scale_y_continuous(labels = scales::label_comma()) +
  labs(y = 'Natural-Origin Spawners',
       x = 'Restoration benefit',
       color = NULL) +
  theme(#legend.position = c(0, 0.98),
    legend.position = "bottom",
    plot.background = element_rect(fill = "white"),
    panel.spacing.y = unit(-0.02, "inches"),
    panel.grid.major = element_line('grey90'),
    plot.title = element_text(size = 10, face = "plain", hjust = 0.5),
    strip.background = element_rect(fill = NA))+
  watermark

ggsave(
  file.path(sens_path, sprintf("chn_sensitivity_STL_noPSM_%s.png", Sys.Date())),
  width = 4,
  height = 4)
