# Script to run scenarios for diagnostics with full LCM outputs

reinstall_harp <- function(){
  devtools::uninstall()
  devtools::install(dependencies = FALSE)
}


# Git status checks ------------------------------------------------------------

# reinstall_harp() # run this if needed to reinstall package
git_status <- harp:::check_git_status()
commit_date <- git_status["commit_date"]
dir_name <- git_status["dir_name"]
cur_branch <- git_status["cur_branch"]


## Prep data directories -------------------------------------------------------

nsims <- 100

# share drive directory
path_to_shared_folder <-
  '//nwcfile.nmfs.local/FE/watershed/Stilly Phase 2 Shared/lcm_results'

if (cur_branch == 'main' & !dir.exists(path_to_shared_folder)) {
  warning('Not connected to the shared drive. Results will not be archived')
}

if(grepl("dirty", dir_name)){
  warning('the repo is dirty. Results will not be archived')}

if(cur_branch == 'main' & dir.exists(path_to_shared_folder)){
  out_path <- file.path(path_to_shared_folder, dir_name)
} else{
  out_path <- paste0('ignore/results/', dir_name)
}

temp_path <- file.path(out_path, "single_scenario", nsims)

try(dir.create(out_path, recursive = TRUE), silent == T)
try(dir.create(temp_path, recursive = TRUE), silent == T)

message(sprintf("Saving results to %s", out_path))

# Packages and parallel setup --------------------------------------------------
library(harp)
library(furrr)
library(tidyverse)
library(progressr)
plan(multisession, workers = 16)
handlers(global = TRUE) # show progress bars

# Scenario Selection -----------------------------------------------------------
diag_scenarios <- c(
  "Current",
  "Barriers",
  "Fine_sediment",
  "Wood",
  "Shade",
  "LR",
  "Beaver",
  "Floodplain",
  "Delta")

# cust_scenarios <- grep("TAG", harp:::scenarios_cust, value = TRUE)
cust_scenarios <- grep("Chinook|TAG", harp:::scenarios_cust, value = TRUE)


scenarios <- rbind(
  expand_grid(
    scenario = c(diag_scenarios, cust_scenarios),
    era = c("curr", "mid", "late")),
  data.frame(scenario = "Historical", era = "hist") # historical
)

# Habitat model ----------------------------------------------------------------

run_hab_model <- function(input_df){ # wrap in function to get status bar
  p <- progressor(steps = nrow(input_df)) # make progress bar
  input_df %>%
    future_pwalk( #parallelize this
      function(scenario, era){
        scenario_clean <- gsub("#", "", scenario) # name without the `#`
        hab_run_stochastic("fall_chinook", scenario, era, runs = 1:nsims) %>%
        saveRDS(
          file.path(
            temp_path,
            sprintf("hab_%s_%s_%s_%s.RDS", scenario_clean, era, nsims, commit_date)))
        p() #increment progress bar
        })}

scenarios %>%
  run_hab_model()

h <- list.files(temp_path, pattern = "hab_", full.names = TRUE) %>%
  map_dfr(readRDS)

saveRDS(
  h,
  file.path(out_path, sprintf('STL_hab_stoch_n%s_%s.rds', nsims, commit_date)),
  compress = 'xz')
