# Script to run scenarios for diagnostics with full LCM outputs
detach(harp)
reinstall_harp <- function(){
  devtools::uninstall()
  devtools::install(dependencies = FALSE)
}


# Git status checks ------------------------------------------------------------

# reinstall_harp() # run this if needed to reinstall package
git_status <- harp:::check_git_status()
commit_date <- git_status["commit_date"]
dir_name <- git_status["dir_name"]
cur_branch <- git_status["cur_branch"]
cur_commit <- git_status["cur_commit"]


## Prep data directories -------------------------------------------------------
nsims <- 100
path_to_shared_folder <-
  '//nwcfile.nmfs.local/FE/watershed/Stilly Phase 2 Shared/lcm_results'

if(cur_branch == 'main' & dir.exists(path_to_shared_folder)){
  out_path <- file.path(path_to_shared_folder, dir_name)
} else{
  out_path <- paste0('ignore/results/', dir_name)
}

data_path <- file.path(out_path, "single_scenario", nsims)
rds_path <- file.path(out_path, "hatchery_rds")
fig_path <- file.path(out_path, "hatchery_figs")

try(dir.create(rds_path), silent = T)

message(sprintf("Saving results to %s", out_path))

# Packages and parallel setup --------------------------------------------------
library(harp)
library(furrr)
library(tidyverse)
plan(multisession, workers = 16)

# Select scenarios and eras ----------------------------------------------------
#target_scenarios <- c("Current", "TAG #1", "Chinook #1a")
target_scenarios <- c("Current")
target_eras <- c("curr", "mid", "late")

scenarios_eras <- expand_grid(scenario = target_scenarios, era =target_eras) %>%
  mutate(file_pattern = sprintf(
    "hab_%s_%s", # pattern
    gsub("#", "", scenario), # fix scenario names
    era))

# Retrieve Habitat Data --------------------------------------------------------

hab_data <- map_dfr( # load data for each combo of scenario and era
  scenarios_eras$file_pattern,
  ~list.files(data_path, pattern = .x, full.names = TRUE) %>%
    readRDS() %>% # read each one in
    lcm_assign() # run lcm_assign() on each
)

hab_data_noPSM <- map_dfr( # load data for each combo of scenario and era
  scenarios_eras$file_pattern,
  ~list.files(data_path, pattern = .x, full.names = TRUE) %>%
    readRDS() %>% # read each one in
    lcm_assign(chin_PSM = FALSE) # run lcm_assign() on each
)

# Lifecycle model --------------------------------------------------------------

adjust_hatchery <- function(hatch_mult_nf, hatch_mult_sf, hab_data){
  hab_data %>%
    mutate(
      nf_release = nf_release * hatch_mult_nf,
      sf_release = sf_release * hatch_mult_sf,
      tot_bs = tot_bs * hatch_mult_nf # scale broodstock with NF release
    ) %>%
    lcm_run(c("spawners_nor", "spawners_hor", "spawners"))}

adjust_hatchery_both <- function(hatch_mult, hab_data){
  adjust_hatchery(hatch_mult, hatch_mult, hab_data) %>%
    mutate(sim = "both", hatch_mult = hatch_mult)}

adjust_hatchery_nf <- function(hatch_mult, hab_data){
  adjust_hatchery(hatch_mult, 1, hab_data) %>%
    mutate(sim = "nf", hatch_mult = hatch_mult)}

adjust_hatchery_sf <- function(hatch_mult, hab_data){
  adjust_hatchery(1, hatch_mult, hab_data) %>%
    mutate(sim = "sf", hatch_mult = hatch_mult)}

adjust_hatchery_all <- function(hatch_mult, hab_data){
  hab_data %>%
    group_split(era, scenario) %>%
    map_dfr(~bind_rows(
      adjust_hatchery_both(hatch_mult, .x),
      adjust_hatchery_nf(hatch_mult, .x),
      adjust_hatchery_sf(hatch_mult, .x)))}


hatch_data <- future_map_dfr(
  seq(0.1, 2,0.1),
  \(x) adjust_hatchery_all(x, hab_data))

saveRDS(
  hatch_data,
  file.path(rds_path, sprintf("hatchery_%s_%s.RDS", nsims, commit_date)))

hatch_data_noPSM <- future_map_dfr(
  seq(0.1, 2,0.1),
  \(x) adjust_hatchery_all(x, hab_data_noPSM))

saveRDS(
  hatch_data_noPSM,
  file.path(rds_path, sprintf("hatchery_noPSM_%s_%s.RDS", nsims, commit_date)))

hatch_data_combined <- tidytable::bind_rows(
  tidytable::mutate(hatch_data, PSM = TRUE),
  tidytable::mutate(hatch_data_noPSM, PSM = FALSE)
)

# Goal-meeting scenario --------------------------------------------------------
reach_goal <- function(hab_data){
  hab_data %>%
    mutate(
      nf_release = 220000,
      sf_release = 200000) %>%
    lcm_run(c("spawners_nor", "spawners_hor", "spawners"))}

hatch_data_goals <- map_dfr(
  group_split(hab_data, era, scenario), # split hab data by era and scenario
  reach_goal)

# Summary stats ----------------------------------------------------------------

subb_grps <- harp:::subbasin_names %>%
  transmute(
    subbasin = noaa_subba,
    subb_group = case_when(
      grepl("North_Fork", subb_group) ~ "North Fork",
      grepl("South_Fork", subb_group) ~ "South Fork",
      TRUE ~ "Lower River"))

hatch_table_forks <- hatch_data_combined %>%
  tidytable::left_join(subb_grps, by = "subbasin") %>%
  tidytable::summarize(
    n = sum(n),
    .by = c(lifestage, era, scenario, year, run, sim, hatch_mult, subb_group, PSM)) %>%
  tidytable::pivot_wider(names_from = lifestage, values_from = n) %>%
  tidytable::mutate(
    era = forcats::fct_relevel(era, "curr", "mid", "late"),
    PHOS = spawners_hor/spawners,
    PNI = if_else(subb_group == "North Fork", 0.5/(0.5+PHOS), 1/(1+PHOS)),
    meets_stnd = (PHOS <= 0.3) & (PNI >= 50),
    perc_BS = case_when(
      subb_group == "North Fork" & sim != "sf" ~ 70*hatch_mult/(spawners_nor + 70*hatch_mult),
      subb_group == "North Fork" & sim == "sf" ~ 70/(spawners_nor + 70),
      TRUE ~ 0))


# find maximum efforts where NF broodstock cost stays below 20%
hatch_table_forks %>%
  tidytable::filter(
    subb_group == "North Fork", sim == "nf", scenario == "Current") %>%
  tidytable::summarise(
    perc_BS = median(perc_BS),
    .by = c("era", "hatch_mult", "PSM")) %>%
  tidytable::mutate(under25 = perc_BS <= 0.2) %>%
  tidytable::summarize(effort = max(hatch_mult[under25]), .by = c(era, PSM))

overharvest <- hatch_table_forks %>%
  tidytable::filter(subb_group == "North Fork") %>%
  tidytable::mutate(overharvest = perc_BS == 1) %>%
  tidytable::group_by(era, scenario, sim, hatch_mult, PSM) %>%
  tidytable::summarise(
    perc_over = sum(overharvest)/n(),
    impossible = sum(overharvest)/n() > 0.3)
# none of these take all the fish >30% of time

hatch_table_big <- hatch_data_combined %>%
  tidytable::summarize( # sum basinwide
    n = sum(n),
    .by = c(lifestage, era, scenario, year, run, sim, hatch_mult, PSM)) %>%
  tidytable::pivot_wider(names_from = lifestage, values_from = n) %>%
  tidytable::mutate(
    era = forcats::fct_relevel(era, "curr", "mid", "late"),
    PHOS = spawners_hor/spawners
    #meets_stnd = (PHOS <= 0.3) & (PNI >= 50)
    ) %>%
  tidytable::left_join(
    overharvest, by = c("era", "scenario", "sim", "hatch_mult", "PSM"))


hatch_medians <- hatch_table_big %>%
  tidytable::group_by(era, scenario, sim, hatch_mult, PSM) %>%
  tidytable::summarize(across(where(is.numeric), median)) %>%
  tidytable::select(-c(year, run))

data.table::fwrite(
  hatch_medians,
  file.path(out_path, sprintf("hatchery_medians_%s.csv", dir_name)))

hatch_data_goals %>%
  tidytable::summarise(
    n = sum(n),
    .by = c(lifestage, era, scenario, year, run, PSM)) %>%
  tidytable::summarise(
    n = median(n),
    .by = c(lifestage, era, scenario, PSM)) %>%
  pivot_wider(names_from = lifestage, values_from = n) %>%
  arrange(scenario, factor(era, levels = c("curr", "mid", "late"))) %>%
  data.table::fwrite(
    file.path(out_path, sprintf("hatchery_goal_medians_%s.csv", cur_commit)))


# Plot results -----------------------------------------------------------------
# annotate with version number
watermark <-  annotate(
  "text", x = Inf, y = -Inf, label = cur_commit,
  hjust=1, vjust=-1.1,  col="black",
  fontface = "italic",
  alpha = 0.2)

custom_facet_theme <- function(scale_type = "free_y"){
  list(
  facet_wrap(
    "era", scales = scale_type, nrow = 1,
    labeller = as_labeller(c(curr = "2020s", mid = "2050s", late = "2080s"))),
  cowplot::theme_cowplot(),
  theme(
    strip.background = element_rect(fill = "white"),
    plot.background = element_rect(fill = "white")))}

## Box plots -------------------------------------------------------------------
hatch_table_forks %>% # find broodstock take for no-action scenario
  filter(
    scenario == "Current",
    PSM,
    sim == "both", hatch_mult == 1, subb_group == "North Fork") %>%
  mutate(era = factor(
    era,
    levels = harp:::eras_all,
    labels = c('1800s', '2020s', '2050s', '2080s'))) %>%
  ggplot()+
  geom_boxplot(
    aes(era, perc_BS, fill = era),
    outlier.size = 0.1, outlier.colour = "#cccccc")+
  scale_fill_manual(
    values = c(
      `2020s` = "#fce67b",
      `2050s`  = scales::hue_pal()(2)[1],
      `2080s` = scales::hue_pal()(2)[2]))+
  labs(
    x = NULL,
    y = "NF Broodstock Take")+
  scale_y_continuous(labels = scales::percent_format(), limits = c(0,1))+
  coord_cartesian(ylim = c(0, 1))+
  cowplot::theme_cowplot()+
  theme(
    #plot.margin = margin(l = 50),
    strip.background = NULL, # no facet strip
    strip.text = element_blank(),
    plot.background = element_rect(fill = "white"), # white bg
    #axis.text.x = element_text(angle = 45, hjust = 1),
    legend.position = "none")+
  watermark

ggsave(
  sprintf("%s/BS_cost_boxplot_PSM_%s.png", fig_path, commit_date),
  width = 6,
  height = 4)

hatch_table_forks %>% # find broodstock take for no-action scenario
  filter(
    scenario == "Current",
    !PSM,
    sim == "both", hatch_mult == 1, subb_group == "North Fork") %>%
  mutate(era = factor(
    era,
    levels = harp:::eras_all,
    labels = c('1800s', '2020s', '2050s', '2080s'))) %>%
  ggplot()+
  geom_boxplot(
    aes(era, perc_BS, fill = era),
    outlier.size = 0.1, outlier.colour = "#cccccc")+
  scale_fill_manual(
    values = c(
      `2020s` = "#fce67b",
      `2050s`  = scales::hue_pal()(2)[1],
      `2080s` = scales::hue_pal()(2)[2]))+
  labs(
    x = NULL,
    y = "NF Broodstock Take")+
  scale_y_continuous(labels = scales::percent_format(), limits = c(0,1))+
  coord_cartesian(ylim = c(0, 1))+
  cowplot::theme_cowplot()+
  theme(
    #plot.margin = margin(l = 50),
    strip.background = NULL, # no facet strip
    strip.text = element_blank(),
    plot.background = element_rect(fill = "white"), # white bg
    #axis.text.x = element_text(angle = 45, hjust = 1),
    legend.position = "none")+
  watermark

ggsave(
  sprintf("%s/BS_cost_boxplot_noPSM_%s.png", fig_path, commit_date),
  width = 6,
  height = 4)


med_curr_hor <- hatch_table_big %>%
  filter(
    PSM,
    sim == "both", hatch_mult == 1,
    era == "curr", scenario == "Current") %>%
  pull(spawners_hor) %>%
  median()

hatch_table_big %>% # move this to the other figure script?
  filter(sim == "both", hatch_mult == 1, scenario == "Current", PSM) %>%
  mutate(era = factor(
    era,
    levels = harp:::eras_all,
    labels = c('1800s', '2020s', '2050s', '2080s'))) %>%
  ggplot()+
  geom_hline(
    yintercept = med_curr_hor,
    color = "#aaaaaa")+
  geom_boxplot(aes(era, spawners_hor, fill = era), outlier.shape=NA)+
  scale_fill_manual(
    values = c(
      `2020s` = "#fce67b",
      `2050s`  = scales::hue_pal()(2)[1],
      `2080s` = scales::hue_pal()(2)[2]))+
  labs(
    x = NULL,
    y = "Hatchery-Origin Spawners")+
  cowplot::theme_cowplot()+
  theme(
    #plot.margin = margin(l = 50),
    strip.background = NULL, # no facet strip
    strip.text = element_blank(),
    plot.background = element_rect(fill = "white"), # white bg
    # axis.text.x = element_text(angle = 45, hjust = 1),
    legend.position = "none")+
  watermark

ggsave(
  sprintf("%s/HOR_boxplot_%s.png", fig_path, commit_date),
  width = 4,
  height = 4)

med_curr_hor_noPSM <- hatch_table_big %>%
  filter(
    PSM == FALSE,
    sim == "both", hatch_mult == 1,
    era == "curr", scenario == "Current") %>%
  pull(spawners_hor) %>%
  median()

hatch_table_big %>% # move this to the other figure script?
  filter(sim == "both", hatch_mult == 1, scenario == "Current", !PSM) %>%
  mutate(era = factor(
    era,
    levels = harp:::eras_all,
    labels = c('1800s', '2020s', '2050s', '2080s'))) %>%
  ggplot()+
  geom_hline(
    yintercept = med_curr_hor_noPSM,
    color = "#aaaaaa")+
  geom_boxplot(aes(era, spawners_hor, fill = era), outlier.shape=NA)+
  scale_fill_manual(
    values = c(
      `2020s` = "#fce67b",
      `2050s`  = scales::hue_pal()(2)[1],
      `2080s` = scales::hue_pal()(2)[2]))+
  labs(
    x = NULL,
    y = "Hatchery-Origin Spawners")+
  cowplot::theme_cowplot()+
  theme(
    #plot.margin = margin(l = 50),
    strip.background = NULL, # no facet strip
    strip.text = element_blank(),
    plot.background = element_rect(fill = "white"), # white bg
    # axis.text.x = element_text(angle = 45, hjust = 1),
    legend.position = "none")+
  watermark

ggsave(
  sprintf("%s/HOR_boxplot_noPSM_%s.png", fig_path, commit_date),
  width = 4,
  height = 4)

hatch_data_goals %>%
  filter(lifestage != "spawners", scenario == "Current", PSM = TRUE) %>%
  tidytable::summarise(n = sum(n), .by = c(lifestage, era, year, run)) %>%
  mutate(era = factor(
    era,
    levels = harp:::eras_all,
    labels = c('1800s', '2020s', '2050s', '2080s'))) %>%
  ggplot()+
  geom_boxplot(aes(era, n, fill = era), outlier.shape=NA, alpha = 0.6)+
  scale_fill_manual(
    values = c(
      `2020s` = "#fce67b",
      `2050s`  = scales::hue_pal()(2)[1],
      `2080s` = scales::hue_pal()(2)[2]))+
  facet_wrap(
    "lifestage",
    #scales = "free_y",
    labeller = as_labeller(c(
      spawners_nor = "Natural-Origin",
      spawners_hor = "Hatchery-Origin")))+
  labs(
    x = NULL,
    y = "Spawners")+
  cowplot::theme_cowplot()+
  theme(
    #plot.margin = margin(l = 50),
    strip.background = NULL, # no facet strip
    #strip.text = element_blank(),
    plot.background = element_rect(fill = "white"), # white bg
    # axis.text.x = element_text(angle = 45, hjust = 1),
    legend.position = "none")+
  watermark

ggsave(
  sprintf("%s/goals_boxplot_%s.png", fig_path, commit_date),
  width = 8,
  height = 4)

## Line plots ------------------------------------------------------------------

make_9plot_quants <- function(simtype, PSM){
  ribbon_color <- switch(simtype,
    nf = "#7FC97F", #scales::brewer_pal(type = "qual")(3)
    sf = "#BEAED4",
    both = "#FDC086")


  big_table_subset <-  hatch_table_big %>%
    filter(sim == simtype, !impossible, scenario == "Current", PSM == .env$PSM)

  hor_75 <- big_table_subset %>%
    filter(hatch_mult == 2, era == "curr") %>%
    pull(spawners_hor) %>%
    quantile(0.75)

  nor_plot <- big_table_subset %>%
    ggplot()+
    geom_vline(xintercept = 1, color = "gray50", size = 0.2)+
    stat_summary(
      aes(hatch_mult, spawners_nor),
      geom = "ribbon",
      fun.max = ~quantile(.x, 0.75),
      fun.min = ~quantile(.x, 0.25),
      fill = ribbon_color,
      alpha = 0.5)+
    stat_summary(
      aes(hatch_mult, spawners_nor),
      geom = "line",
      fun = median)+
    labs(
      x = "Hatchery Operation Intensity",
      y = "Natural-Origin Spawners",
      # y = "Spawners",
      color = "Spawner\nOrigin")+
    scale_x_continuous(labels = scales::label_percent())+
    scale_y_continuous(labels = scales::label_comma())+
    custom_facet_theme() +
    watermark

  hor_plot <- big_table_subset %>%
    ggplot()+
    geom_vline(xintercept = 1, color = "gray50", size = 0.2)+
    stat_summary(
      aes(hatch_mult, spawners_hor),
      geom = "ribbon",
      fun.max = ~quantile(.x, 0.75),
      fun.min = ~quantile(.x, 0.25),
      fill = ribbon_color,
      alpha = 0.5)+
    stat_summary(
      aes(hatch_mult, spawners_hor),
      geom = "line",
      fun = median)+
    labs(
      x = "Hatchery Operation Intensity",
      y = "Hatchery-Origin Spawners",
      # y = "Spawners",
      color = "Spawner\nOrigin")+
    scale_x_continuous(labels = scales::label_percent())+
    scale_y_continuous(labels = scales::label_comma())+
    coord_cartesian(ylim = c(0, hor_75))+
    custom_facet_theme() +
    watermark


  phos_plot <- big_table_subset %>%
    ggplot()+
    geom_vline(xintercept = 1, color = "gray50", size = 0.2)+
    stat_summary(
      aes(hatch_mult, PHOS),
      geom = "ribbon",
      fun.max = ~quantile(.x, 0.75),
      fun.min = ~quantile(.x, 0.25),
      fill = ribbon_color,
      alpha = 0.5)+
    stat_summary(
      aes(hatch_mult, PHOS),
      geom = "line",
      fun = median)+
    labs(
      x = "Hatchery Operation Intensity",
      y = "Basinwide pHOS")+
    scale_x_continuous(labels = scales::label_percent())+
    scale_y_continuous(labels = scales::label_percent())+
    coord_cartesian(ylim = c(0, 0.8))+
    custom_facet_theme()

  bs_plot <- hatch_table_forks %>%
    tidytable::filter(PSM == .env$PSM) %>%
    dplyr::left_join(
      overharvest, by = join_by(era, scenario, sim, hatch_mult, PSM)) %>%
    filter(
      sim == simtype,
      !impossible, subb_group == "North Fork",
      scenario == "Current") %>%
    ggplot()+
    geom_vline(xintercept = 1, color = "gray50", linewidth = 0.2)+
    stat_summary(
      aes(hatch_mult, perc_BS),
      geom = "ribbon",
      fun.max = ~quantile(.x, 0.75),
      fun.min = ~quantile(.x, 0.25),
      fill = ribbon_color,
      alpha = 0.5)+
    stat_summary(
      aes(hatch_mult, perc_BS),
      geom = "line",
      fun = median)+
    labs(
      x = "Hatchery Operation Intensity",
      y = "NF Broodstock Take")+
    scale_x_continuous(labels = scales::label_percent())+
    scale_y_continuous(labels = scales::label_percent())+
    coord_cartesian(ylim = c(0, 1))+
    custom_facet_theme()

  spawner_plot <- cowplot::plot_grid(
    nor_plot+labs(x = NULL), hor_plot+labs(x = NULL), bs_plot,
    ncol = 1,
    align = "v", axis = "lr")

  cowplot::save_plot( # save the 9-panel plots
    sprintf(
      "%s/9panel_hatch_adj%s_PSM%s_%s.png",
      fig_path, simtype, PSM, commit_date),
    spawner_plot,
    base_width = 8,
    base_height = 8)

  cowplot::save_plot( # save the 3-panel plots
    sprintf(
      "%s/3panel_hatch_sp_adj%s_PSM%s_%s.png",
      fig_path, simtype, PSM, commit_date),
    nor_plot,
    base_width = 8,
    base_height = 2.5)

  cowplot::save_plot( # save the 3-panel plots
    sprintf(
      "%s/3panel_hatch_ho_adj%s_PSM%s_%s.png",
      fig_path, simtype, PSM, commit_date),
    hor_plot,
    base_width = 8,
    base_height = 2.5)

  cowplot::save_plot( # phos
    sprintf(
      "%s/3panel_hatch_ph_adj%s_PSM%s_%s.png",
      fig_path, simtype, PSM, commit_date),
    phos_plot,
    base_width = 8,
    base_height = 2.5)

  cowplot::save_plot( # broodstock cost
    sprintf("%s/3panel_hatch_bs_adj%s_PSM%s_%s.png",
            fig_path, simtype, PSM, commit_date),
    bs_plot,
    base_width = 8,
    base_height = 2.5)
}

make_9plot_quants("both", PSM = TRUE)
make_9plot_quants("both", PSM = FALSE)
make_9plot_quants("nf", PSM = TRUE)
make_9plot_quants("nf", PSM = FALSE)
make_9plot_quants("sf", PSM = TRUE)
make_9plot_quants("sf", PSM = FALSE)

hatch_table_forks %>%
  tidytable::filter(sim == "both", subb_group == "North Fork", PSM) %>%
  ggplot()+
  geom_vline(xintercept = 1, color = "gray50", size = 0.2)+
  geom_hline(yintercept = 0.5, color = "gray80", size = 0.2)+
  stat_summary(
    aes(hatch_mult, PNI, fill = subb_group),
    geom = "ribbon",
    fun.max = ~quantile(.x, 0.75),
    fun.min = ~quantile(.x, 0.25),
    alpha = 0.5)+
  stat_summary(
    aes(hatch_mult, PNI, color = subb_group),
    geom = "line",
    fun = median)+
  labs(
    x = "Hatchery Operation Intensity",
    y = "North Fork PNI",
    fill = "Location",
    color = "Location")+
  scale_x_continuous(labels = scales::label_percent())+
  scale_y_continuous(labels = scales::label_percent())+
  coord_cartesian(ylim = c(0.3, 0.9))+
  scale_color_brewer(type = "qual")+
  scale_fill_brewer(type = "qual")+
  # facet_grid(subb_group~era)
  custom_facet_theme()+
  theme(legend.position = "none")+
  watermark

ggsave(
  sprintf("%s/PNI_qrt_%s.png", fig_path, commit_date),
  width = 8,
  height = 4)

hatch_table_forks %>%
  tidytable::filter(sim == "both", subb_group == "North Fork", !PSM) %>%
  ggplot()+
  geom_vline(xintercept = 1, color = "gray50", size = 0.2)+
  geom_hline(yintercept = 0.5, color = "gray80", size = 0.2)+
  stat_summary(
    aes(hatch_mult, PNI, fill = subb_group),
    geom = "ribbon",
    fun.max = ~quantile(.x, 0.75),
    fun.min = ~quantile(.x, 0.25),
    alpha = 0.5)+
  stat_summary(
    aes(hatch_mult, PNI, color = subb_group),
    geom = "line",
    fun = median)+
  labs(
    x = "Hatchery Operation Intensity",
    y = "North Fork PNI",
    fill = "Location",
    color = "Location")+
  scale_x_continuous(labels = scales::label_percent())+
  scale_y_continuous(labels = scales::label_percent())+
  coord_cartesian(ylim = c(0.3, 0.9))+
  scale_color_brewer(type = "qual")+
  scale_fill_brewer(type = "qual")+
  # facet_grid(subb_group~era)
  custom_facet_theme()+
  theme(legend.position = "none")+
  watermark

ggsave(
  sprintf("%s/PNI_qrt_noPSM_%s.png", fig_path, commit_date),
  width = 8,
  height = 4)

# hatch_table_big %>%
#   filter(sim == "both", !impossible, PSM) %>%
#   ggplot()+
#   geom_vline(xintercept = 1, color = "gray50", size = 0.2)+
#   stat_summary(
#     aes(hatch_mult, spawners_nor, color = scenario),
#     geom = "line",
#     fun = median)+
#   labs(
#     x = "Hatchery Operation Intensity",
#     y = "Natural-Origin Spawners",
#     # y = "Spawners",
#     color = "Scenario")+
#   scale_x_continuous(labels = scales::label_percent())+
#   scale_y_continuous(labels = scales::label_comma())+
#   scale_color_brewer(type = "qual", palette = 2)+
#   custom_facet_theme() +
#   theme(legend.position = "bottom")+
#   watermark
#
# ggsave( # HOR spawners
#   sprintf("%s/3panel_hatch_NOR_scenarios_%s.png", fig_path, commit_date),
#   width = 8,
#   height = 2.7)




