# Script to run scenarios for diagnostics with full LCM outputs

reinstall_harp <- function(){
  devtools::uninstall()
  devtools::install(dependencies = FALSE)
}


# Git status checks ------------------------------------------------------------

# reinstall_harp() # run this if needed to reinstall package
git_status <- harp:::check_git_status()
commit_date <- git_status["commit_date"]
dir_name <- git_status["dir_name"]
cur_branch <- git_status["cur_branch"]


## Prep data directories -------------------------------------------------------

nsims <- 100

# share drive directory
path_to_shared_folder <-
  '//nwcfile.nmfs.local/FE/watershed/Stilly Phase 2 Shared/lcm_results'

if (cur_branch == 'main' & !dir.exists(path_to_shared_folder)) {
  warning('Not connected to the shared drive. Results will not be archived')
}

if(grepl("dirty", dir_name)){
  warning('the repo is dirty. Results will not be archived')}

if(cur_branch == 'main' & dir.exists(path_to_shared_folder)){
  out_path <- file.path(path_to_shared_folder, dir_name)
} else{
  out_path <- paste0('ignore/results/', dir_name)
}

temp_path <- file.path(out_path, "single_scenario", nsims)

try(dir.create(out_path, recursive = TRUE), silent == T)
try(dir.create(temp_path, recursive = TRUE), silent == T)

message(sprintf("Saving results to %s", out_path))

# Packages and parallel setup --------------------------------------------------
library(harp)
library(furrr)
library(tidyverse)
library(progressr)
plan(multisession, workers = 16)
handlers(global = TRUE) # show progress bars

# Lifecycle model --------------------------------------------------------------

read_hab <- function(hab_path){ # function to read one habitat data file
  readRDS(hab_path)
}

create_lcm <- function(df_hab, chin_PSM = TRUE){ # function to run one lcm
  era <- unique(df_hab$era)
  scenario <- unique(df_hab$scenario)
  psm_flag <- if(chin_PSM){"PSM"}else{"noPSM"}
  hatch_mult_nf <- tidytable::case_when(
    # different maximum NF hatchery efforts based on era and PSM assumptions
    era == "hist" ~ 1, # clearly unrealistic. Better would be to set both to 0.
    chin_PSM & era == "curr" ~ 1,
    chin_PSM & era =="mid" ~ 0.7,
    chin_PSM & era == "late" ~ 0.3,
    !chin_PSM & era == "curr" ~ 1,
    !chin_PSM & era =="mid" ~ 1,
    !chin_PSM & era == "late" ~ 0.9)
  cat(paste(era, scenario, hatch_mult_nf, "\n"))
  lcm_data <- lcm_assign(df_hab, chin_PSM = chin_PSM) %>%
    mutate(
      nf_release = nf_release * hatch_mult_nf,
      tot_bs = tot_bs * hatch_mult_nf) %>%
    lcm_run("all")
  saveRDS(
    lcm_data,
    file.path(
      temp_path,
      sprintf("lcm_%s_%s_%s_%s_%s.RDS", psm_flag, scenario, era, nsims, commit_date)))
  lcm_data
}

# function to run batch of lcms in parallel with progress bar
run_batch_lcm <- function(path_list, chin_PSM = TRUE){
  p <- progressor(along = path_list)
  psm_flag <- if(chin_PSM){"PSM"}else{"noPSM"}
  save_path <-file.path(
    out_path,
    sprintf('STL_lcm_stoch_%s_%s_%s.rds', nsims, psm_flag, commit_date))
  path_list |>
    future_map_dfr(\(x){
      lcm_data <- read_hab(x) |>
        create_lcm(chin_PSM = chin_PSM)
      p()
      lcm_data}) |>
    saveRDS(save_path, compress = 'xz') # save it
}

h_list <- list.files( # list habitat data
  temp_path,
  pattern = sprintf("hab_.+%s.RDS", commit_date),
  full.names = TRUE)

h_list |> run_batch_lcm() # run with PSM

h_list |> run_batch_lcm(chin_PSM = FALSE) # run wihout PSM
