#Script to run scenarios for diagnostics with full LCM outputs

reinstall_harp <- function(){
  devtools::uninstall()
  devtools::install(dependencies = FALSE)
}


# Git status checks ------------------------------------------------------------

# reinstall_harp() # run this if needed to reinstall package
git_status <- harp:::check_git_status()
commit_date <- git_status["commit_date"]
dir_name <- git_status["dir_name"]
cur_branch <- git_status["cur_branch"]
cur_tag <- git_status["cur_tag"]

## Prep data directories -------------------------------------------------------

nsims <- 100

# share drive directory
path_to_shared_folder <-
  '//nwcfile.nmfs.local/FE/watershed/Stilly Phase 2 Shared/lcm_results'

if (cur_branch == 'main' & !dir.exists(path_to_shared_folder)) {
  warning('Not connected to the shared drive. Results will not be archived')
}

if(grepl("dirty", dir_name)){
  warning('the repo is dirty. Results will not be archived')}

if(cur_branch == 'main' & dir.exists(path_to_shared_folder)){
  out_path <- file.path(path_to_shared_folder, dir_name)
} else{
  out_path <- paste0('ignore/results/', dir_name)
}

fig_path <- file.path(out_path, "scenario_figs")

temp_path <- file.path(out_path, "single_scenario", nsims)

try(dir.create(out_path, recursive = TRUE), silent == T)
try(dir.create(fig_path, recursive = TRUE), silent == T)

message(sprintf("Saving results to %s", out_path))
message(sprintf("Saving figures to %s", fig_path))

# Packages ---------------------------------------------------------------------
library(harp)
library(tidyverse)

# Convenience funs -------------------------------------------------------------
diag_scenarios <- c(
  "Current",
  "Barriers",
  "Fine_sediment",
  "Wood",
  "Shade",
  "LR",
  "Beaver",
  "Floodplain",
  "Delta")

cust_scenarios <- grep("Chinook|TAG", harp:::scenarios_cust, value = TRUE)

scenarios <- c(diag_scenarios, cust_scenarios)

fix_names <- function(x){x %>% str_replace('_', ' ') %>% tools::toTitleCase()}

# factorize pops
fix_pops <- function(x) {factor(x, levels = pops_all, labels = pops_fixed)}

# factorize scenarios
fix_scenarios <- function(x){
  factor(x, levels = c("Historical", diag_scenarios, cust_scenarios))}

fix_eras <- function(x) { # factorize eras
  factor(
    x,
    levels = harp:::eras_all,
    labels = c('1800s', '2020s', '2050s', '2080s'))}

# Summary Stats ----------------------------------------------------------------

make_summ_stats <- function(search_pattern, save_label){
  lcm_data <- list.files( # list lcm data
    temp_path,
    pattern = sprintf(search_pattern),
    full.names = TRUE) %>%
    tidytable::map_dfr(readRDS)

  chinook_medians <- lcm_data %>%
    tidytable::summarize(
      n = sum(n),
      .by = c(scenario, era, lifestage, year, run)) %>%
    tidytable::filter(
      lifestage %in% c("spawners_nor", "spawners_hor"),
      scenario != "Historical") %>%
    tidytable::summarize(
      n = median(n),
      .by = c(scenario, lifestage, era)) %>% # median of time series
    dplyr::mutate( # difference from current
      n_cur = n[scenario == 'Current' & era == 'curr'],
      diff_perc = (n - n_cur)/n_cur,
      diff_str = scales::percent(
        na_if(diff_perc, 0),
        accuracy = 1, style_positive = "plus"),
      scenario = fix_scenarios(scenario),
      .by = lifestage) %>%
    dplyr::mutate( # difference from era-specific no action
      n_NoAc = n[scenario == "Current"],
      diff_perc_NoAc = (n-n_NoAc)/n_NoAc,
      diff_str_NoAc = scales::percent(
        na_if(diff_perc_NoAc, 0),
        accuracy = 1, style_positive = "plus"),
      .by = c(era, lifestage)
    )

  data.table::fwrite(
    chinook_medians,
    file.path(
      out_path,
      sprintf("chinook_medians_%s_n%s_%s.csv", save_label, nsims, commit_date)))

  chinook_basin_ts <- lcm_data %>%
    tidytable::summarize(
      n = sum(n),
      .by = c(scenario, era, lifestage, year, run)) %>%
    tidytable::filter(
      lifestage %in% c("spawners_nor", "spawners_hor"),
      era != "hist") %>%
    tidytable::mutate(
      scenario = fix_scenarios(scenario),
      era = fix_eras(era))

  med_curr <- chinook_medians %>%
    tidytable::filter(scenario == 'Current', era == 'curr')

  return(list(
    chinook_medians = chinook_medians,
    chinook_basin_ts = chinook_basin_ts,
    med_curr = med_curr))
}

stats_PSM   <- make_summ_stats(
  sprintf("lcm_PSM.+_%s.RDS", commit_date), # pattern to find results with
  "PSM") # label for saved csv
stats_noPSM <- make_summ_stats(
  sprintf("lcm_noPSM.+_%s.RDS", commit_date),
  "noPSM")

saveRDS(stats_PSM, sprintf("%s/stats_PSM_n%s.RDS", out_path, nsims))
saveRDS(stats_noPSM, sprintf("%s/stats_noPSM_n%s.RDS", out_path, nsims))

# Plotting functions------------------------------------------------------------

top_pal <- harp:::scenario_desc %>%
  filter(scenario %in% diag_scenarios) %>%
  pull(plot_color)

pop_names <-c(
  `coho` = "Coho",
  `fall_chinook` = "Chinook",
  `steelhead` = "Steelhead",
  `wsteelhead` = "Winter Steelhead",
  `ssteelhead` = "Summer Steelhead")

scen_names <-c(
  `Current` = "No Action",
  `Barriers` = "No Barriers",
  `Fine_sediment` = "Historical Fine Sediment",
  `Barriers` = "No Manmade Barriers",
  `Wood` = "Historic Wood Loading",
  `Shade` = "Historic Shade",
  `LR` = "Bank Armor Removal",
  `Beaver` = "Historic Beaver Ponds",
  `Floodplain` = "Historic Floodplain Habitat",
  `Delta` = "Historic Estuary Habitat",
  setNames(cust_scenarios, cust_scenarios))

#reusable code to make barplots
make_barplot <- function(df){
  df %>%
    filter(lifestage == "spawners_nor") %>%
    ggplot() +
    cowplot::theme_half_open() +
    geom_bar(
      aes(scenario, n, fill = scenario),
      color = 'white',
      stat = 'identity') +
    geom_bar(
      aes(scenario, n_cur),
      fill = "gray",
      linewidth = 0.7,
      alpha = 0.7,
      color = 'white',
      stat = 'identity') +
    geom_text(
      aes(scenario,n, label = diff_str),
      vjust = -0.2,
      alpha = 0.8,
      color = 'black') +
    labs(x = NULL, y = "Natural-Origin Spawners", fill = NULL) +
    scale_y_continuous(
      labels = scales::comma,
      expand = expansion(c(0, 0.1)))+
    theme(
      # plot.margin = margin(l = 50),
      panel.grid.major = element_line(color = 'gray90'),
      panel.grid.major.x = element_blank(),
      axis.text.x = element_text(angle = 45, hjust = 1),
      strip.background = element_blank(),
      plot.background = element_rect(fill = "white"),
      legend.position = 'none')}


# function to make barplots of diagnostic scenarios
make_diag_barplot <- function(df){
  df %>%
    filter(scenario %in% diag_scenarios) %>%
    mutate(scenario = factor(scenario, levels = diag_scenarios)) %>%
    make_barplot()+
    scale_fill_manual(values = top_pal) +
    scale_x_discrete(label = as_labeller(scen_names))}

make_custom_barplot <- function(df){
  df %>%
    filter(
      scenario %in% cust_scenarios) %>%
    make_barplot()}

# annotate with version number
watermark <-  annotate(
  "text", x = Inf, y = -Inf, label = cur_tag,
  hjust=1, vjust=-1.1,  col="black",
  fontface = "italic",
  alpha = 0.2)

# Make specific plots for the report--------------------------------------------

make_diag_2020s <- function(stats_list, save_label){
  diag_plot <- stats_list$chinook_medians %>%
    filter(era == "curr") %>%
    make_diag_barplot()+
    watermark

  ggsave(
    sprintf(
      "%s/barplot_STL_chn_stoch_%s_%s.png",
      fig_path, save_label, commit_date),
    diag_plot,
    width = 6, height = 4)

  return(diag_plot)}

make_diag_2020s(stats_PSM, "PSM")
make_diag_2020s(stats_noPSM, "noPSM")

make_cust_2020s <- function(stats_list, save_label){
  cust_plot <- stats_list$chinook_medians %>%
    mutate(
      scenario = factor(
        scenario,
        levels = c(
          "TAG #1", "TAG #2",
          "Chinook #1a", "Chinook #2a"))) %>%
    filter(era == "curr") %>%
    make_custom_barplot() +
    scale_fill_manual(
      #scales::brewer_pal(palette = "Paired")(8)[c(1:4,7:8)]
      values = c("#A6CEE3","#1F78B4","#B2DF8A","#33A02C","#FDBF6F","#FF7F00"))+
    watermark


  ggsave(
    sprintf(
      "%s/barplot_STL_chn_stoch_cust_a_%s_%s.png",
      fig_path, save_label, commit_date),
    cust_plot,
    width = 3, height = 4)

  return(cust_plot)}

make_cust_2020s(stats_PSM, "PSM")
make_cust_2020s(stats_noPSM, "noPSM")


# Basinwide boxplots -----------------------------------------------------------
focus_scen<-c(
  "Current",
  "Wood",
  "Shade",
  "LR",
  "Floodplain",
  cust_scenarios)

## climate change boxplot ------------------------------------------------------
make_cc_box_nor <- function(stats_list, save_label){
  ymax <- stats_noPSM$chinook_basin_ts |>
    tidytable::filter(scenario == "Current", lifestage == "spawners_nor") |>
    tidytable::pull(n) |>
    max()

  cc_nor <- stats_list$chinook_basin_ts %>%
    filter(
      scenario == "Current", lifestage == "spawners_nor") %>%
    ggplot()+
    geom_hline(
      yintercept = stats_list$med_curr$n[1],
      color = "#aaaaaa")+
    geom_boxplot(
      aes(era, n, fill = era),
      outlier.size = 0.1, outlier.colour = "#cccccc")+
    labs(x = NULL, y = "Natural-Origin Spawners", fill = NULL)+ #no labels
    scale_y_continuous(
      limits = c(0, ymax),
      labels = scales::comma,
      expand = expansion(c(0, 0)))+
    scale_fill_manual(
      values = c(
        `2020s` = "#fce67b",
        `2050s`  = scales::hue_pal()(2)[1],
        `2080s` = scales::hue_pal()(2)[2]))+
    cowplot::theme_cowplot()+
    theme(
      strip.background = NULL, # no facet strip
      strip.text = element_blank(),
      plot.background = element_rect(fill = "white"), # white bg
      legend.position = "none")+
    watermark

  ggsave(
    sprintf(
      "%s/boxplot_STL_chn_basinwide_by_era_NoAct_%s_%s.png",
      fig_path, save_label, commit_date),
    cc_nor,
    width = 4, height = 4)

  return(cc_nor)
  }
make_cc_box_nor(stats_PSM, "PSM")
make_cc_box_nor(stats_noPSM, "noPSM")

make_cc_box <- function(stats_list, save_label){
  ymax <- stats_noPSM$chinook_basin_ts |>
    tidytable::filter(scenario == "Current") |>
    tidytable::pull(n) |>
    max()

  cc_box <- stats_list$chinook_basin_ts %>%
    filter(
      scenario == "Current") %>%
    ggplot()+
    geom_hline(
      aes(yintercept = n),
      data = stats_list$med_curr,
      color = "#aaaaaa")+
    geom_boxplot(
      aes(era, n, fill = era),
      outlier.size = 0.1, outlier.colour = "#cccccc")+
    labs(x = NULL, y = "Spawners", fill = NULL)+ #no labels
    scale_y_continuous(
      limits = c(0, ymax),
      labels = scales::comma,
      expand = expansion(c(0, 0)))+
    facet_wrap(
      "lifestage",
      labeller = as_labeller(c(
        spawners_nor = "Natural-Origin",
        spawners_hor = "Hatchery-Origin")))+
    scale_fill_manual(
      values = c(
        `2020s` = "#fce67b",
        `2050s`  = scales::hue_pal()(2)[1],
        `2080s` = scales::hue_pal()(2)[2]))+
    cowplot::theme_cowplot()+
    theme(
      strip.background = NULL, # no facet strip
      plot.background = element_rect(fill = "white"), # white bg
      legend.position = "none")+
    watermark


  ggsave(
    sprintf(
      "%s/boxplot_STL_chn_basinwide_by_era_by_origin_%s_%s.png",
      fig_path, save_label, commit_date),
    cc_box,
    width = 8, height = 4)

  cc_box
}

make_cc_box(stats_PSM, "PSM")
make_cc_box(stats_noPSM, "noPSM")

## Scenario barplots -----------------------------------------------------------

make_climate_barplot <- function(scenario_vec, stats_list){
  stats_list$chinook_medians %>%
    filter(
      scenario %in% scenario_vec,
      lifestage == "spawners_nor") %>%
    mutate(
      era = fix_eras(era),
      b = grepl("b", scenario)) %>%
    ggplot()+
    geom_bar(
      aes(scenario, n, fill = era, alpha = b),
      color = 'white',
      stat = 'identity') +
    geom_bar(
      aes(scenario, n_NoAc),
      fill = "gray",
      linewidth = 0.7,
      alpha = 0.7,
      color = 'white',
      stat = 'identity') +
    geom_text(
      aes(scenario,n, label = diff_str_NoAc),
      vjust = -0.2,
      size = 3,
      alpha = 0.8,
      color = 'black') +
    scale_alpha_manual(
      values = c(1, 0.3),
      breaks = c(FALSE, TRUE))+
    labs(x = NULL, y = "Natural-Origin Spawners", fill = NULL)+ #no labels
    scale_y_continuous(
      labels = scales::comma,
      expand = expansion(c(0, 0.1)))+
    scale_fill_manual(
      values = c(
        `2020s` = "#fce67b",
        `2050s`  = scales::hue_pal()(2)[1],
        `2080s` = scales::hue_pal()(2)[2]))+
    facet_wrap("era", scales = "fixed", nrow = 1)+
    scale_x_discrete(label = as_labeller(scen_names))+
    cowplot::theme_cowplot()+
    theme(
      plot.margin = margin(l = 30),
      panel.grid.major.x = element_blank(),
      axis.text.x = element_text(angle = 45, hjust = 1),
      strip.background = element_blank(),
      plot.background = element_rect(fill = "white"),
      legend.position = 'none')+
    watermark
}

make_top4_bars <- function(stats_list, save_label){
  top4_bar <- make_climate_barplot(
    c("Wood", "Shade", "LR", "Floodplain"),
    stats_list)
  ggsave(
    sprintf(
      "%s/barplot_STL_chn_diagnostic_eras_%s_%s.png",
      fig_path, save_label, commit_date),
    top4_bar,
    width = 5.5, height = 4.5)
  return(top4_bar)
}

make_top4_bars(stats_PSM, "PSM")
make_top4_bars(stats_noPSM, "noPSM")

make_cust_bars_cc <- function(stats_list, save_label){
  cust_bar <- make_climate_barplot(
    cust_scenarios,
    stats_list)
  ggsave(
    sprintf(
      "%s/barplot_STL_chn_custom_eras_bs_%s_%s.png",
      fig_path, save_label, commit_date),
    width = 8, height = 4.5)
  return(cust_bar)
}

make_cust_bars_cc(stats_PSM, "PSM")
make_cust_bars_cc(stats_noPSM, "noPSM")
