# script to make summary tables for Stilly Phase 2 report

# Make summary plots for chinook stochastic model outputs
library(tidytable)
library(ggplot2)

# Git status checks ------------------------------------------------------------

# reinstall_harp() # run this if needed to reinstall package
git_status <- harp:::check_git_status()
commit_date <- git_status["commit_date"]
dir_name <- git_status["dir_name"]
cur_branch <- git_status["cur_branch"]

## Prep data directories -------------------------------------------------------
nsims <- 100
path_to_shared_folder <-
  '//nwcfile.nmfs.local/FE/watershed/Stilly Phase 2 Shared/lcm_results'

if(cur_branch == 'main' & dir.exists(path_to_shared_folder)){
  out_path <- file.path(path_to_shared_folder, dir_name)
} else{
  out_path <- paste0('ignore/results/', dir_name)
}

data_path <- file.path(out_path, "single_scenario", nsims)

try(dir.create(rds_path), silent = T)

message(sprintf("Saving results to %s", out_path))


# Find data --------------------------------------------------------------------

lcm_rds <- list.files(
  data_path,
  pattern = sprintf('lcm_.*_Current_.*_%s_', nsims), # only get No Action scen.
  full.names = TRUE)

# Read the data ----------------------------------------------------------------
# lifestages for table
lstages <- c("eggs", "pre_fry", "fry_migrants", "parr_migrants",
             "spawners_nor", "spawners_hor")

read_NoAc_rds <- function(rdspath){ # function to read and format RDS files
  PSM_use <- grepl("lcm_PSM_Current", rdspath) # did the model include PSM?
  readRDS(rdspath) %>%
    filter(lifestage %in% lstages) %>%
    mutate(PSM = PSM_use)
}

# read in LCM data
lcm_NoAc <- tidytable::map_dfr(lcm_rds, read_NoAc_rds)

# sum abundances across subbasins, then get median across all year x run combos
NoAc_sum <- lcm_NoAc %>%
  filter(lifestage %in% lstages) %>%
  summarise(
    n = sum(n),
    .by = c(pop, scenario, era, run, year, lifestage, PSM))

NoAc_medians <- NoAc_sum %>%
  summarise(
    n = median(n),
    .by = c(pop, scenario, era, lifestage, PSM))

# calculate percent change
NoAc_change <- NoAc_medians %>%
  dplyr::mutate(
    n_curr = n[era == "curr"],
    .by = c(pop, scenario, lifestage, PSM)) %>%
  filter(era != "curr") %>%
  mutate(
    n_diff = n_curr - n,
    perc_diff = n_diff/n_curr,
    perc_format = scales::label_percent(1)(perc_diff))

data.table::fwrite(
  NoAc_change,
  sprintf("%s/No_Action_PercChange_detailed_%s.csv", out_path, commit_date))

NoAc_change %>%
  mutate(
    modeltype = factor(
      PSM,
      levels = c(TRUE, FALSE),
      labels = c("Model 1 (PSM)", "Model 2 (No PSM)")),
    era = factor(
      era,
      levels = c("mid", "late"),
      labels = c("2050s", "2080s")),
    lifestage = factor(
      lifestage,
      levels = lstages,
      labels = c("Eggs", "Emergent fry", "Fry migrants", "Parr migrants",
                 "NOR spawners", "HOR spawners"))) %>%
  select(pop, scenario, era, lifestage, modeltype, perc_format) %>%
  pivot_wider(names_from = c(modeltype, era), values_from = perc_format) %>%
  data.table::fwrite(
    sprintf("%s/No_Action_PercChange_simple_%s.csv", out_path, commit_date))

data.table::fwrite(
  NoAc_change,
  sprintf("%s/No_Action_PercChange_detailed_%s.csv", out_path, commit_date))
