# SAR diagnostics

# Git status checks ------------------------------------------------------------

# reinstall_harp() # run this if needed to reinstall package
git_status <- harp:::check_git_status()
dir_name <- git_status["dir_name"]
cur_branch <- git_status["cur_branch"]
cur_tag <- git_status["cur_tag"]
commit_date <- git_status["commit_date"]


# Setup ------------------------------------------------------------------------
library(tidyverse)
library(furrr)
library(harp)
plan(multisession, workers = 16)

nsims <- 100 # number of simulations to look for

## Prep data directories -------------------------------------------------------

# share drive directory
path_to_shared_folder <-
  '//nwcfile.nmfs.local/FE/watershed/Stilly Phase 2 Shared/lcm_results'

if (cur_branch == 'main' & !dir.exists(path_to_shared_folder)) {
  warning('Not connected to the shared drive. Results will not be archived')
}

if(grepl("dirty", dir_name)){
  warning('the repo is dirty. Results will not be archived')}

if(cur_branch == 'main' & dir.exists(path_to_shared_folder)){
  out_path <- file.path(path_to_shared_folder, dir_name)
} else{
  out_path <- paste0('ignore/results/', dir_name)
}

rds_dir <- file.path(out_path, "SAR_rds")
fig_dir <- file.path(out_path, "SAR_figs")
hab_dir <- file.path(out_path, "single_scenario", nsims)

try(dir.create(rds_dir, recursive = TRUE), silent == T)

message(sprintf("Saving results to %s", out_path))

# Load in habitat data ---------------------------------------------------------
# function to read and lcm-assign a habitat RDS
read_assign <- function(x){
  file_name <- sprintf(x, nsims, commit_date)
  readRDS(file.path(hab_dir, file_name)) %>% lcm_assign()}

# to do- figure out 80% and 20% delta restoration values

NoAct_curr <- read_assign("hab_Current_curr_%s_%s.RDS")
NoAct_late <- read_assign("hab_Current_late_%s_%s.RDS")
TAG2_curr <- read_assign("hab_TAG 2_curr_%s_%s.RDS")
TAG2_late <- read_assign("hab_TAG 2_late_%s_%s.RDS")
TAG1_curr <- read_assign("hab_TAG 1_curr_%s_%s.RDS")
TAG1_late <- read_assign("hab_TAG 1_late_%s_%s.RDS")

# make delta scenarios by adapting no-action scenarios
Delta20_curr <- NoAct_curr %>% # TAG 1 used 20% delta
  tidytable::mutate(
    capacity_delta = TAG1_curr$capacity_delta,
    scenario = "Estuary 20%")

Delta20_late <- NoAct_late %>%
  tidytable::mutate(
    capacity_delta = TAG1_curr$capacity_delta,
    scenario = "Estuary 20%")

Delta80_curr <- NoAct_curr %>% # TAG 2 used 80% delta
  tidytable::mutate(
    capacity_delta = TAG2_curr$capacity_delta,
    scenario = "Estuary 80%")

Delta80_late <- NoAct_late %>%
  tidytable::mutate(
    capacity_delta = TAG2_curr$capacity_delta,
    scenario = "Estuary 80%")

# Simulation -------------------------------------------------------------------
adjust_ps_parr <- function(marine_adj, base_df){
  base_df %>%
    mutate(
      ps_parr_surv = ps_parr_surv*marine_adj,
      ps_yr_surv = ps_yr_surv*marine_adj,
      sw_surv_h = sw_surv_h*marine_adj) %>%
    lcm_run(c(
      "parr_migrants", "fry_migrants",
      "spawners_nor",
      'total_run_nor', 'total_run_subyr',
      'spawners_nor', 'spawners')) %>%
    tidytable::summarise(
      n = sum(n),
      .by = c(lifestage, run, year, scenario, era)) %>%
    tidytable::pivot_wider(names_from = lifestage, values_from = n) %>%
    tidytable::transmute(
      scenario, era,
      parr_migrants, fry_migrants,
      marine_adj = marine_adj,
      spawners_nor,
      spawners,
      smolt = parr_migrants+fry_migrants,
      adult_sy = spawners_nor * (total_run_subyr/total_run_nor),
      SAR_sy = (adult_sy+70)/smolt)}

adjust_parr_range <- function(base_df){
  future_map_dfr( # parallelize this step
    seq(1, 35, 1),
    ~adjust_ps_parr(.x, base_df))}

# run what-if simulations on all habitat data frames
lifestage_all <- map_dfr(
  list( # all of the data frames
    NoAct_curr,
    NoAct_late,
    TAG1_curr,
    TAG1_late,
    TAG2_curr,
    TAG2_late,
    Delta20_curr,
    Delta20_late,
    Delta80_curr,
    Delta80_late),
  adjust_parr_range)

saveRDS(lifestage_all, file.path(rds_dir, "SAR_sims_all.RDS"))

lifestage_medians <-lifestage_all %>%
  tidytable::summarise(
    tidytable::across(where(is.numeric), median),
    .by = c(marine_adj, scenario, era)) %>%
  mutate(
    SAR_NoAc = SAR_sy[scenario == "Current"],
    n_diff = spawners_nor - spawners_nor[scenario == "Current"],
    .by = c(marine_adj, era)) %>%
  mutate(scenario = factor(
    scenario,
    levels = c("Current", "TAG #1", "TAG #2", "Estuary 20%", "Estuary 80%"),
    labels = c("No Action", "TAG #1", "TAG #2", "Estuary 20%", "Estuary 80%")
  ))

saveRDS(lifestage_medians, file.path(rds_dir, "SAR_sims_medians.RDS"))

# Base plotting functions ------------------------------------------------------
watermark <-  annotate(
  "text", x = Inf, y = -Inf, label = cur_tag,
  hjust=1, vjust=-1.1,  col="black",
  fontface = "italic",
  alpha = 0.2)

scenario_aesthetics <- list(
  scale_color_manual(
    name = NULL,
    breaks = c("No Action", "TAG #1", "TAG #2", "Estuary 20%", "Estuary 80%"),
    values = c("#cccccc","#A6CEE3","#1F78B4","#FDBF6F", "#FF7F00")),
  scale_linetype_manual(
    name = NULL,
    breaks = c("No Action", "TAG #1", "TAG #2", "Estuary 20%", "Estuary 80%"),
    values = c(2, rep(1,4)))
)

plot_nor_v_SAR <- function(medians_data){
  medians_data %>%
    ggplot()+
    geom_line(aes(
      SAR_NoAc,
      spawners_nor,
      color = scenario,
      linetype = scenario))+
    scale_y_continuous(
      labels = scales::label_comma(),
      limits = c(0, max(lifestage_medians$spawners_nor)),
      expand = c(0,0))+
    scale_x_continuous(labels = scales::label_percent())+
    scenario_aesthetics+
    labs(x = "SAR (No Action equivalent)", y = "Natural-Origin Spawners")+
    facet_wrap(
      "era",
      labeller = as_labeller(c(curr = "2020s", late = "2080s")))+
    cowplot::theme_cowplot()+
    theme(
      strip.background = NULL, # no facet strip
      # strip.text = element_blank(),
      plot.background = element_rect(fill = "white"), # white bg
      legend.position = "bottom")+
    watermark}

plot_nor_extra_v_SAR <- function(medians_data){
  medians_data %>%
    filter(scenario != "No Action") %>%
    ggplot()+
    geom_line(aes(
      SAR_NoAc,
      n_diff,
      color = scenario))+
    scale_y_continuous(
      labels = scales::label_comma(),
      limits = c(0, max(lifestage_medians$n_diff)),
      expand = c(0,0))+
    scale_x_continuous(labels = scales::label_percent())+
    scenario_aesthetics+
    labs(
      x = "SAR (No Action equivalent)",
      y = "Additional Natural-Origin Spawners")+
    facet_wrap(
      "era",
      labeller = as_labeller(c(curr = "2020s", late = "2080s")))+
    cowplot::theme_cowplot()+
    theme(
      strip.background = NULL, # no facet strip
      # strip.text = element_blank(),
      plot.background = element_rect(fill = "white"), # white bg
      legend.position = "bottom")+
    watermark}

save_SAR_plot <- function(filename, width = 7, height = 4, ...){
  ggsave(file.path(fig_dir, filename), width = width, height = height, ...)}

# Zoomed out plots -------------------------------------------------------------

c(
  "#A6CEE3", "#1F78B4",
  "#B2DF8A", "#33A02C",
  "#FB9A99", "#E31A1C",
  "#FDBF6F", "#FF7F00",
  "#CAB2D6", "#6A3D9A") # extra colors

lifestage_medians %>%
  plot_nor_v_SAR()

save_SAR_plot("NOR_vs_SAR_all.png")

lifestage_medians %>%
  tidytable::filter(!grepl("Estuary", scenario)) %>%
  plot_nor_v_SAR()

save_SAR_plot("NOR_vs_SAR_TAG.png")

lifestage_medians %>%
  tidytable::filter(!grepl("TAG", scenario))%>%
  plot_nor_v_SAR()

save_SAR_plot("NOR_vs_SAR_Delta.png")


# Zoomed in plots --------------------------------------------------------------


lifestage_medians %>%
  plot_nor_v_SAR()+
  coord_cartesian(
    xlim = c(min(lifestage_medians$SAR_sy), 0.008),
    ylim = c(0, 25000))

save_SAR_plot("NOR_vs_SAR_all_zoom.png")

lifestage_medians %>%
  tidytable::filter(!grepl("Estuary", scenario)) %>%
  plot_nor_v_SAR()+
  coord_cartesian(
    xlim = c(min(lifestage_medians$SAR_sy), 0.008),
    ylim = c(0, 25000))

save_SAR_plot("NOR_vs_SAR_TAG_zoom.png")

lifestage_medians %>%
  tidytable::filter(!grepl("TAG", scenario)) %>%
  plot_nor_v_SAR()+
  coord_cartesian(
    xlim = c(min(lifestage_medians$SAR_sy), 0.008),
    ylim = c(0, 25000))

save_SAR_plot("NOR_vs_SAR_Delta_zoom.png")


# extra spawners plot ----------------------------------------------------------

lifestage_medians %>%
  plot_nor_extra_v_SAR()

save_SAR_plot("NORplus_vs_SAR_all.png")

lifestage_medians %>%
  tidytable::filter(scenario != "No Action") %>%
  plot_nor_extra_v_SAR()+
  coord_cartesian(
    xlim = c(min(lifestage_medians$SAR_sy), 0.008),
    ylim = c(0, 8000))

save_SAR_plot("NORplus_vs_SAR_all_zoom.png")

lifestage_medians %>%
  tidytable::filter(!grepl("Estuary", scenario)) %>%
  plot_nor_extra_v_SAR()

save_SAR_plot("NORplus_vs_SAR_TAG.png")

lifestage_medians %>%
  tidytable::filter(!grepl("TAG", scenario)) %>%
  plot_nor_extra_v_SAR()

save_SAR_plot("NORplus_vs_SAR_Delta.png")


# fry plot ---------------------------------------------------------------------
lifestage_medians %>%
  ggplot()+
  geom_line(aes(
    SAR_NoAc,
    fry_migrants,
    color = scenario,
    linetype = scenario))+
  scale_y_continuous(labels = scales::label_comma())+
  scale_x_continuous(labels = scales::label_percent())+
  scenario_aesthetics+
  labs(x = "SAR (No Action equivalent)", y = "Fry Migrants")+
  facet_wrap("era", labeller = as_labeller(c(curr = "2020s", late = "2080s")))+
  cowplot::theme_cowplot()+
  theme(
    strip.background = NULL, # no facet strip
    # strip.text = element_blank(),
    plot.background = element_rect(fill = "white"), # white bg
    legend.position = "bottom")

save_SAR_plot("fry_vs_SAR.png")


# spawner plot with scenario SAR -----------------------------------------------
lifestage_medians %>%
  ggplot()+
  geom_line(aes(
    SAR_sy,
    spawners_nor,
    color = scenario))+
  geom_point(aes(
    SAR_sy,
    spawners_nor,
    color = scenario))+
  scenario_aesthetics+
  scale_y_continuous(labels = scales::label_comma())+
  scale_x_continuous(labels = scales::label_percent())+
  labs(x = "SAR", y = "Natural-Origin Spawners")+
  facet_wrap("era", labeller = as_labeller(c(curr = "2020s", late = "2080s")))+
  # coord_cartesian(xlim = c(0.004, 0.008), ylim = c(0, 20000))+
  cowplot::theme_cowplot()+
  theme(
    strip.background = NULL, # no facet strip
    # strip.text = element_blank(),
    plot.background = element_rect(fill = "white"), # white bg
    legend.position = "bottom")

save_SAR_plot("fry_vs_scenario_SAR.png")
